import os
path = os.path
from random import randrange

from myhdl import (block, Signal, intbv, delay, instance, StopSimulation)
from myhdl.conversion import verify
import pytest

NRTESTS = 10


@block
def binaryOps(
    Bitand,
# #               Bitor,
# #               Bitxor,
# #               FloorDiv,
    LeftShift,
    Modulo,
    Mul,
# #               Pow,
    RightShift,
    Sub,
    Sum, Sum1, Sum2, Sum3,
    EQ,
    NE,
    LT,
    GT,
    LE,
    GE,
    BoolAnd,
    BoolOr,
    left, right, aBit):

    @instance
    def comb():
        while 1:
            yield left, right, aBit
    # #         Bitand.next = left & right
    # #         Bitor.next = left | right
    # #         Bitxor.next = left ^ right
    # #         if right != 0:
    # #             FloorDiv.next = left // right
            # Keep left shifts smaller than 2** 31 for VHDL's to_integer
            LeftShift.next = 0
            if left < 256 and right < 22 and right >= 0:
                LeftShift.next = left << right
    # #         if right != 0:
    # #             Modulo.next = left % right
            Mul.next = left * right
    # #         # Icarus doesn't support ** yet
    # #         #if left < 256 and right < 22:
    # #         #    Pow.next = left ** right
    # #         Pow.next = 0
    # #         if right >= -0:
    # #            RightShift.next = left >> right
                # # RightShift.next = left
            Sub.next = left - right
            Sum.next = left + right
            Sum1.next = left + right[2:]
            Sum2.next = left + right[1]
            Sum3.next = left + aBit
            EQ.next = left == right
            NE.next = left != right
            LT.next = left < right
            GT.next = left > right
            LE.next = left <= right
            GE.next = left >= right
            BoolAnd.next = bool(left) and bool(right)
            BoolOr.next = bool(left) or bool(right)

    return comb


@block
def binaryBench(Ll, Ml, Lr, Mr):

    seqL = []
    seqR = []
    for __ in range(NRTESTS):
        seqL.append(randrange(Ll, Ml))
        seqR.append(randrange(Lr, Mr))
    for j, k in ((Ll, Lr), (Ml - 1, Mr - 1), (Ll, Mr - 1), (Ml - 1, Lr)):
        seqL.append(j)
        seqR.append(k)
    seqL = tuple(seqL)
    seqR = tuple(seqR)

    aBit = Signal(bool(0))
    left = Signal(intbv(Ll, min=Ll, max=Ml))
    right = Signal(intbv(Lr, min=Lr, max=Mr))
    M = 2 ** 14

    Bitand = Signal(intbv(0, min=-2 ** 17, max=2 ** 17))
# #        Bitand_v = Signal(intbv(0, min=-2**17, max=2**17))
# #         Bitor = Signal(intbv(0)[max(m, n):])
# #         Bitor_v = Signal(intbv(0)[max(m, n):])
# #         Bitxor = Signal(intbv(0)[max(m, n):])
# #         Bitxor_v = Signal(intbv(0)[max(m, n):])
# #         FloorDiv = Signal(intbv(0)[m:])
# #         FloorDiv_v = Signal(intbv(0)[m:])
    LeftShift = Signal(intbv(0, min=-2 ** 64, max=2 ** 64))
    Modulo = Signal(intbv(0)[M:])
    Mul = Signal(intbv(0, min=-2 ** 17, max=2 ** 17))
# #         Pow = Signal(intbv(0)[64:])
    RightShift = Signal(intbv(0, min=-M, max=M))
    Sub = Signal(intbv(min=-M, max=M))
    Sum, Sum1, Sum2, Sum3 = [Signal(intbv(min=-M, max=M)) for __ in range(4)]
    EQ, NE, LT, GT, LE, GE = [Signal(bool()) for __ in range(6)]
    BoolAnd, BoolOr = [Signal(bool()) for __ in range(2)]

    binops = binaryOps(
        Bitand,
# #                            Bitor,
# #                            Bitxor,
# #                            FloorDiv,
        LeftShift,
        Modulo,
        Mul,
   # #                            Pow,
        RightShift,
        Sub,
        Sum, Sum1, Sum2, Sum3,
        EQ,
        NE,
        LT,
        GT,
        LE,
        GE,
        BoolAnd,
        BoolOr,
        left, right, aBit)

    @instance
    def stimulus():
        for i in range(len(seqL)):
            left.next = seqL[i]
            right.next = seqR[i]
            yield delay(10)

    @instance
    def check():
        while 1:
            yield left, right
            aBit.next = not aBit
            yield delay(1)

                # print "%s %s %s %s" % (left, right, Mul, Mul_v)
                # print "%s %s %s %s" % (left, right, bin(Mul), bin(Mul_v))
                # print "%s %s %s %s" % (left, right, Sum, Sum_v)
                # print "%s %s %s %s" % (left, right, bin(Sum), bin(Sum_v))
# #                 print left
# #                 print right
# #                 print bin(left)
# #                 print bin(right)
# #                 print bin(Bitand)
# #                 print bin(Bitand_v)
# #                 print Bitand
# #                 print Bitand_v
# #                self.assertEqual(Bitand, Bitand_v)
                # w = len(Bitand)
                # self.assertEqual(bin(Bitand, w), bin(Bitand_v,w ))
# #                 self.assertEqual(Bitor, Bitor_v)
# #                 self.assertEqual(Bitxor, Bitxor_v)
# # ##                 self.assertEqual(FloorDiv, FloorDiv_v)
            print(LeftShift)
            # print Modulo
            print(Mul)
                # self.assertEqual(Pow, Pow_v)
            print(RightShift)
            print(Sub)
            print(Sum)
            print(Sum1)
            print(Sum2)
            print(Sum3)
            print(int(EQ))
            print(int(NE))
            print(int(LT))
            print(int(GT))
            print(int(LE))
            print(int(GE))
            print(int(BoolAnd))
            print(int(BoolOr))

    return binops, stimulus, check


@pytest.mark.filterwarnings("ignore:Signal is not driven")
def testBinaryOps():
    for Ll, Ml, Lr, Mr in (
                            (-254, 236, 0, 4),
                            (-128, 128, -128, 128),
                            (-53, 25, -23, 123),
                            (-23, 145, -66, 12),
                            (23, 34, -34, -16),
                            (-54, -20, 45, 73),
                            (-25, -12, -123, -66),
                           ):
        assert binaryBench(Ll, Ml, Lr, Mr).verify_convert() == 0


@block
def unaryOps(
             BoolNot,
             Invert,
             UnaryAdd,
             UnarySub,
             arg):

    @instance
    def comb():
        while 1:
            yield arg
            # BoolNot.next = not arg
            Invert.next = ~arg
            # UnaryAdd.next = +arg
            UnarySub.next = - -arg

    return comb


@block
def unaryBench(m):

    M = 2 ** m
    seqM = tuple([i for i in range(-M, M)])

    arg = Signal(intbv(0, min=-M, max=+M))
    BoolNot = Signal(bool(0))
    Invert = Signal(intbv(0, min=-M, max=+M))
    UnaryAdd = Signal(intbv(0, min=-M, max=+M))
    UnarySub = Signal(intbv(0, min=-M, max=+M))

    unaryops = unaryOps(
                         BoolNot,
                         Invert,
                         UnaryAdd,
                         UnarySub,
                         arg)

    @instance
    def stimulus():
        for i in range(len(seqM)):
            arg.next = seqM[i]
            yield delay(10)
        raise StopSimulation

    @instance
    def check():
        while 1:
            yield arg
            yield delay(1)
            # print BoolNot
            print(Invert)
            # print UnaryAdd
            print(UnarySub)

    return unaryops, stimulus, check


def testUnaryOps():
    for m in (4, 7):
        assert unaryBench(m).verify_convert() == 0


@block
def augmOps(
# #               Bitand,
# #               Bitor,
# #               Bitxor,
# #               FloorDiv,
              LeftShift,
# #               Modulo,
              Mul,
              RightShift,
              Sub,
              Sum,
              left, right):

    M = 2 ** 17
    N = 2 ** 64

    @instance
    def comb():
        var = intbv(0, min=-M, max=+M)
        var2 = intbv(0, min=-N, max=+N)
        while 1:
            yield left, right
    # #         var[:] = left
    # #         var &= right
    # #         Bitand.next = var
    # #         var[:] = left
    # #         var |= right
    # #         Bitor.next = var
    # #         var[:] = left
    # #         var ^= left
    # #         Bitxor.next = var
    # #         if right != 0:
    # #             var[:] = left
    # #             var //= right
    # #             FloorDiv.next = var
            LeftShift.next = 0
            if left < 256 and right < 22 and right >= 0:
                var2[:] = left
                var2 <<= right
                LeftShift.next = var2
    # #         if right != 0:
    # #             var[:] = left
    # #             var %= right
    # #             Modulo.next = var
            var[:] = left
            var *= right
            Mul.next = var

            var[:] = left
            RightShift.next = 0
            if right >= 0:
                var >>= right
                RightShift.next = var

            var[:] = left
            var -= right
            Sub.next = var
            var[:] = left
            var += right
            Sum.next = var

    return comb


@block
def augmBench(Ll, Ml, Lr, Mr):

    M = 2 ** 17

    seqL = []
    seqR = []
    for i in range(NRTESTS):
        seqL.append(randrange(Ll, Ml))
        seqR.append(randrange(Lr, Mr))
    for j, k in ((Ll, Lr), (Ml - 1, Mr - 1), (Ll, Mr - 1), (Ml - 1, Lr)):
        seqL.append(j)
        seqR.append(k)
    seqL = tuple(seqL)
    seqR = tuple(seqR)

    left = Signal(intbv(Ll, min=Ll, max=Ml))
    right = Signal(intbv(Lr, min=Lr, max=Mr))

# #         Bitand = Signal(intbv(0)[max(m, n):])
# #         Bitor = Signal(intbv(0)[max(m, n):])
# #         Bitxor = Signal(intbv(0)[max(m, n):])

# #         FloorDiv = Signal(intbv(0)[m:])
    LeftShift = Signal(intbv(0, min=-2 ** 64, max=2 ** 64))
# #         Modulo = Signal(intbv(0)[m:])

    Mul = Signal(intbv(0, min=-M, max=+M))

    RightShift = Signal(intbv(0, min=-M, max=+M))

    Sub = Signal(intbv(0, min=-M, max=+M))
    Sum = Signal(intbv(0, min=-M, max=+M))

    augmops = augmOps(
# #                            Bitand,
# #                            Bitor,
# #                            Bitxor,
# #                            FloorDiv,
        LeftShift,
# #                            Modulo,
        Mul,
        RightShift,
        Sub,
        Sum,
        left, right)

    @instance
    def stimulus():
        for i in range(len(seqL)):
            left.next = seqL[i]
            right.next = seqR[i]
            yield delay(10)

    @instance
    def check():
        while 1:
            yield left, right
            yield delay(1)
                # print "%s %s %s %s" % (left, right, Or, Or_v)
# #                 self.assertEqual(Bitand, Bitand_v)
# #                 self.assertEqual(Bitor, Bitor_v)
# #                 self.assertEqual(Bitxor, Bitxor_v)
# #                 self.assertEqual(FloorDiv, FloorDiv_v)
            print(LeftShift)
# #                 self.assertEqual(Modulo, Modulo_v)
            print(Mul)
            print(RightShift)
            print(Sub)
            print(Sum)

    return augmops, stimulus, check


def checkAugmOps(Ll, Ml, Lr, Mr):
    assert verify(augmBench, Ll, Ml, Lr, Mr) == 0


def testAugmOps():
    for Ll, Ml, Lr, Mr in (
                            (-254, 236, 0, 4),
                            (-128, 128, -128, 128),
                            (-53, 25, -23, 123),
                            (-23, 145, -66, 12),
                            (23, 34, -34, -16),
                            (-54, -20, 45, 73),
                            (-25, -12, -123, -66),
                           ):
        assert augmBench(Ll, Ml, Lr, Mr).verify_convert() == 0


@block
def expressions(a, b, clk):

    c = Signal(intbv(0, min=0, max=47))

    @instance
    def comb():

        d = intbv(0, min=-23, max=43)
        d[:] = -17

        c.next = 5
        yield clk.posedge
        a.next = c + 1
        b.next = c + 1
        yield clk.posedge
        a.next = c + -10
        b.next = c + -1
        yield clk.posedge
        a.next = c < -10
        b.next = c < -1
        yield clk.posedge
        a.next = d + c
        b.next = d >= c
        yield clk.posedge
# #         a.next = d & c
# #         b.next = c + (d & c)
        yield clk.posedge
        a.next = d + -c
        b.next = c + (-d)
        yield clk.posedge
        a.next = -d
        yield clk.posedge
        a.next = -c
        yield clk.posedge
        c.next = 46
        yield clk.posedge
        a.next = ~d + 1
        b.next = ~c + 1
        yield clk.posedge
        a.next = ~c + 1
        b.next = ~d + 1

        yield clk.posedge
        raise StopSimulation

    return comb


@block
def expressionsBench():

    a = Signal(intbv(0, min=-34, max=47))
    b = Signal(intbv(0, min=0, max=47))
    clk = Signal(bool())

    expr = expressions(a, b, clk)

    @instance
    def check():
        while 1:
            yield clk.posedge
            yield delay(1)
            print(int(a))
            print(int(b))

    @instance
    def clkgen():
        while True:
            yield delay(10)
            clk.next = not clk

    return expr, check, clkgen


def testExpressions():
    assert expressionsBench().verify_convert() == 0




import unittest
from unittest import TestCase
import random
from random import randrange
random.seed(2)

from myhdl import Simulation, StopSimulation, Signal, \
                  delay, intbv, negedge, posedge, now

from dff import dff
from dff_clkout import dff_clkout

ACTIVE_LOW, INACTIVE_HIGH = 0, 1

class TestDff(TestCase):
    
    vals = [randrange(2) for i in range(1000)]

    def clkGen(self, clk):
        while 1:
            yield delay(10)
            clk.next = not clk
            
    
    def stimulus(self, d, clk, reset):
        reset.next = ACTIVE_LOW
        yield negedge(clk)
        reset.next = INACTIVE_HIGH
        for v in self.vals:
            d.next = v
            yield negedge(clk)
        raise StopSimulation
    
    
    def check(self, q, clk, reset):
        yield posedge(reset)
        v_Z = 0
        first = 1
        for v in self.vals:
            yield posedge(clk)
            if not first:
                self.assertEqual(q, v_Z)
            first = 0
            yield delay(3)
            self.assertEqual(q, v)
            v_Z = v
            

    def bench(self):
        
        q, d, clk, reset = [Signal(intbv(0)) for i in range(4)]
        
        DFF_1 = dff(q, d, clk, reset)
        CLK_1 = self.clkGen(clk)
        ST_1 = self.stimulus(d, clk, reset)
        CH_1 = self.check(q, clk, reset)
        
        sim = Simulation(DFF_1, CLK_1, ST_1, CH_1)
        return sim
    

    def test1(self):
        """ dff test """
        sim = self.bench()
        sim.run(quiet=1)
        
        
    def test2(self):
        """ dff test with simulation suspends """
        sim = self.bench()
        while sim.run(duration=randrange(1,5), quiet=1):
            pass
        

    def bench_clkout(self):
        
        clkout = Signal(intbv(0))
        q = Signal(intbv(0), delay=1)
        d = Signal(intbv(0))
        clk = Signal(intbv(0))
        
        reset = Signal(intbv(0))
        DFF_1 = dff_clkout(clkout, q, d, clk, reset)
        CLK_1 = self.clkGen(clk)
        ST_1 = self.stimulus(d, clkout, reset)
        CH_1 = self.check(q, clkout, reset)
        
        sim = Simulation(DFF_1, CLK_1, ST_1, CH_1)
        return sim

    
    def test1_clkout(self):
        """ dff_clkout test """
        sim = self.bench_clkout()
        sim.run(quiet=1)
        

    def test2_clkout(self):
        """ dff_clkout test with simulation suspends """
        sim = self.bench_clkout()
        while sim.run(duration=randrange(1,5), quiet=1):
            pass
    

if __name__ == '__main__':
    unittest.main()


            
            

    

    
        


                

        




import pytest

from myhdl import (block, Signal, ResetSignal, intbv, always_seq, always_comb,
                   instance, delay, StopSimulation,)


class Intf(object):

    def __init__(self):
        self.x = Signal(intbv(1, min=-1111, max=1111))
        self.y = Signal(intbv(2, min=-2211, max=2211))
        self.z = Signal(intbv(3, min=-3311, max=3311))


@block
def modify(clock, reset, a):

    intfa = Intf()

    @always_seq(clock.posedge, reset=reset)
    def rtl_inc():
        intfa.x.next = intfa.x + 1
        intfa.y.next = intfa.y + 2
        intfa.z.next = intfa.z + 3

    @always_comb
    def rtl_add():
        a.x.next = intfa.x + 1
        a.y.next = intfa.y + 2
        a.z.next = intfa.z + 3

    return rtl_inc, rtl_add


@block
def use_interfaces(clock, reset, a, b, c):

    intfa = Intf()
    intfaa = Intf()

    mod_inst = modify(clock, reset, intfaa)

    @always_seq(clock.posedge, reset=reset)
    def rtl_inc():
        intfa.x.next = intfa.x - 1
        intfa.y.next = intfa.y - 2
        intfa.z.next = intfa.z - 3

        b.x.next = b.x + 1
        b.y.next = b.y + 2
        b.z.next = b.z + 3

        c.x.next = c.x + 1
        c.y.next = c.y + 2
        c.z.next = c.z + 3

    @always_comb
    def rtl_combine():
        a.x.next = intfaa.x + 1
        a.y.next = intfaa.y + 2
        a.z.next = intfaa.z + 3

    return mod_inst, rtl_inc, rtl_combine


@block
def name_conflict_after_replace(clock, reset, a, a_x):
    a_x_0 = [Signal(intbv(0)[len(a_x):]) for _ in range(8)]

    @always_seq(clock.posedge, reset=reset)
    def comb():
        a.x.next = a_x
        a_x.next = a_x_0[1]

    return comb


@pytest.mark.xfail
def test_name_conflict_after_replace():
    clock = Signal(False)
    reset = ResetSignal(0, active=0, isasync=False)
    a = Intf()
    a_x = Signal(intbv(0)[len(a.x):])
    inst = name_conflict_after_replace(clock, reset, a, a_x)
    assert inst.analyze_convert() == 0


@block
def c_testbench():
    clock = Signal(bool(0))
    reset = ResetSignal(0, active=0, isasync=False)
    a, b, c = Intf(), Intf(), Intf()

    tb_dut = use_interfaces(clock, reset, a, b, c)

    @instance
    def tb_clk():
        clock.next = False
        yield delay(10)
        while True:
            clock.next = not clock
            yield delay(10)

    @instance
    def tb_stim():
        reset.next = False
        yield delay(23)
        reset.next = True
        yield delay(33)
        for ii in range(17):
            print("a: x=%d y=%d z=%d" % (a.x, a.y, a.z,))
            print("b: x=%d y=%d z=%d" % (b.x, b.y, b.z,))
            print("c: x=%d y=%d z=%d" % (c.x, c.y, c.z,))
            yield clock.posedge

        raise StopSimulation

    return tb_dut, tb_clk, tb_stim


def test_name_conflicts_analyze():
    clock = Signal(bool(0))
    reset = ResetSignal(0, active=0, isasync=False)
    a, b, c = Intf(), Intf(), Intf()
    inst = use_interfaces(clock, reset, a, b, c)
    assert inst.analyze_convert() == 0


def test_name_conflicts_verify():
    inst = c_testbench()
    assert inst.verify_convert() == 0


if __name__ == '__main__':
    clock = Signal(False)
    reset = ResetSignal(0, active=0, isasync=False)
    a = Intf()
    a_x = Signal(intbv(0)[len(a.x):])
    inst = name_conflict_after_replace(clock, reset, a, a_x)

    inst.convert()
    inst.convert(hdl='VHDL')




import myhdl
from myhdl import *

from glibc_random import glibc_random


def max2(z, a, b):

    @always_comb
    def comb():
        if a > b:
            z.next = a
        else:
            z.next = b

    return comb


def maxn(z, a):
    L = len(a)
    assert L
    assert L % 2 == 0
    H = L // 2
    W = len(a[0])
    if L == 2:
        comp2 = max2(z, a[1], a[0])
        return comp2
    else:
        zlo = Signal(intbv(0)[W:])
        zhi = Signal(intbv(0)[W:])
        complo = maxn(zlo, a[H:])
        comphi = maxn(zhi, a[:H])
        comp2 = max2(z, zhi, zlo)
        return comp2, complo, comphi


def test_findmax():

    L = 32
    W = 16

    clock = Signal(bool())
    stopped = Signal(bool())

    a0 = Signal(intbv(0)[W:])
    a1 = Signal(intbv(0)[W:])
    a2 = Signal(intbv(0)[W:])
    a3 = Signal(intbv(0)[W:])
    a4 = Signal(intbv(0)[W:])
    a5 = Signal(intbv(0)[W:])
    a6 = Signal(intbv(0)[W:])
    a7 = Signal(intbv(0)[W:])
    a8 = Signal(intbv(0)[W:])
    a9 = Signal(intbv(0)[W:])
    a10 = Signal(intbv(0)[W:])
    a11 = Signal(intbv(0)[W:])
    a12 = Signal(intbv(0)[W:])
    a13 = Signal(intbv(0)[W:])
    a14 = Signal(intbv(0)[W:])
    a15 = Signal(intbv(0)[W:])
    a16 = Signal(intbv(0)[W:])
    a17 = Signal(intbv(0)[W:])
    a18 = Signal(intbv(0)[W:])
    a19 = Signal(intbv(0)[W:])
    a20 = Signal(intbv(0)[W:])
    a21 = Signal(intbv(0)[W:])
    a22 = Signal(intbv(0)[W:])
    a23 = Signal(intbv(0)[W:])
    a24 = Signal(intbv(0)[W:])
    a25 = Signal(intbv(0)[W:])
    a26 = Signal(intbv(0)[W:])
    a27 = Signal(intbv(0)[W:])
    a28 = Signal(intbv(0)[W:])
    a29 = Signal(intbv(0)[W:])
    a30 = Signal(intbv(0)[W:])
    a31 = Signal(intbv(0)[W:])

    a = [
         a0,
         a1 ,
         a2 ,
         a3 ,
         a4 ,
         a5 ,
         a6 ,
         a7 ,
         a8 ,
         a9 ,
         a10,
         a11,
         a12,
         a13,
         a14,
         a15,
         a16,
         a17,
         a18,
         a19,
         a20,
         a21,
         a22,
         a23,
         a24,
         a25,
         a26,
         a27,
         a28,
         a29,
         a30,
         a31
         ]

    z = Signal(intbv(0)[W:])

    dut = maxn(z, a)

    @instance
    def clockgen():
        clock.next = 0
        yield delay(10)
        while not stopped:
            clock.next = not clock
            yield delay(10)

    @instance
    def stimulus():
        stopped.next = 0
        exp = intbv(0)[W:]
        val = intbv(0)[W:]
        v = [intbv(0)[W:] for i in range(L)]
        random_word = intbv(0)[32:]
        random_word[:] = 93
        for i in range(2 ** 18):
            exp[:] = 0
            for s in range(L):
                random_word[:] = glibc_random(random_word)
                val[:] = random_word[W:]
                if exp < val:
                    exp[:] = val
                v[s][:] = val

            a0.next = v[0]
            a1.next = v[1]
            a2.next = v[2]
            a3.next = v[3]
            a4.next = v[4]
            a5.next = v[5]
            a6.next = v[6]
            a7.next = v[7]
            a8.next = v[8]
            a9.next = v[9]
            a10.next = v[10]
            a11.next = v[11]
            a12.next = v[12]
            a13.next = v[13]
            a14.next = v[14]
            a15.next = v[15]
            a16.next = v[16]
            a17.next = v[17]
            a18.next = v[18]
            a19.next = v[19]
            a20.next = v[20]
            a21.next = v[21]
            a22.next = v[22]
            a23.next = v[23]
            a24.next = v[24]
            a25.next = v[25]
            a26.next = v[26]
            a27.next = v[27]
            a28.next = v[28]
            a29.next = v[29]
            a30.next = v[30]
            a31.next = v[31]

            yield clock.negedge
            assert z == exp
        stopped.next = 1
        yield delay(10)

    return dut, clockgen, stimulus


if __name__ == '__main__':
    sim = Simulation(test_findmax())
    sim.run()



import os
path = os.path

from myhdl import block, Signal, intbv, delay, instance, always_comb


@block
def bin2gray2(B, G, width):
    """ Gray encoder.

    B -- input intbv signal, binary encoded
    G -- output intbv signal, gray encoded
    width -- bit width
    """

    @instance
    def comb():
        Bext = intbv(0)[width + 1:]
        while 1:
            yield B
            Bext[:] = B
            for i in range(width):
                G.next[i] = Bext[i + 1] ^ Bext[i]

    return comb


@block
def bin2gray(B, G, width):

    """ Gray encoder.

    B -- input intbv signal, binary encoded
    G -- output intbv signal, gray encoded
    width -- bit width

    """

    @always_comb
    def comb():
        Bext = intbv(0)[width + 1:]
        Bext[:] = B
        for i in range(width):
            G.next[i] = Bext[i + 1] ^ Bext[i]

    return comb


@block
def bin2grayBench(width, bin2gray):

    B = Signal(intbv(0)[width:])
    G = Signal(intbv(0)[width:])

    bin2gray_inst = bin2gray(B, G, width)

    n = 2 ** width

    @instance
    def stimulus():
        for i in range(n):
            B.next = i
            yield delay(10)
            # print "B: " + bin(B, width) + "| G_v: " + bin(G_v, width)
            # print bin(G, width)
            # print bin(G_v, width)
            print("%d" % G)

    return stimulus, bin2gray_inst


def test1():
    assert bin2grayBench(width=8, bin2gray=bin2gray).verify_convert() == 0


def test2():
    assert bin2grayBench(width=8, bin2gray=bin2gray2).verify_convert() == 0



from myhdl import (block, Signal, intbv, delay, always_comb, instance)


@block
def map_case4(z, a):

    @always_comb
    def comb():
        if a == 0:
            z.next = 0
        elif a == 1:
            z.next = 1
        elif a == 2:
            z.next = 2
        else:
            z.next = 3

    return comb


@block
def map_case2(z, a):

    @always_comb
    def comb():
        z.next = 0
        if a == 0:
            z.next = 0
        elif a == 1:
            z.next = 1

    return comb


@block
def map_case3(z, a):

    @always_comb
    def comb():
        if a == 0:
            z.next = 0
        elif a == 1:
            z.next = 1
        else:
            z.next = 2

    return comb


@block
def map_case4_full(z, a):

    @always_comb
    def comb():
        if a == 0:
            z.next = 0
        elif a == 1:
            z.next = 1
        elif a == 2:
            z.next = 2
        elif a == 3:
            z.next = 3

    return comb


@block
def bench_case(map_case, N):

    a = Signal(intbv(0)[2:])
    z = Signal(intbv(0)[2:])

    inst = map_case(z, a)

    @instance
    def stimulus():
        for i in range(N):
            a.next = i
            yield delay(10)
            print(z)

    return stimulus, inst


@block
def bool_bench_case(map_case):

    a = Signal(False)
    z = Signal(intbv(0)[2:])

    inst = map_case(z, a)

    @instance
    def stimulus():
        for i in range(2):
            a.next = i
            yield delay(10)
            print(z)

    return stimulus, inst


@block
def length1_bench_case(map_case):

    a = Signal(intbv(0)[1:])
    z = Signal(intbv(0)[2:])

    inst = map_case(z, a)

    @instance
    def stimulus():
        for i in range(2):
            a.next = i
            yield delay(10)
            print(z)

    return stimulus, inst


def test_case4():
    assert bench_case(map_case4, 4).verify_convert() == 0


def test_case2():
    assert bench_case(map_case2, 2).verify_convert() == 0


def test_case3():
    assert bench_case(map_case3, 3).verify_convert() == 0


def test_case4_full():
    assert bench_case(map_case4_full, 4).verify_convert() == 0


def test_case2_bool():
    assert bool_bench_case(map_case3).verify_convert() == 0


def test_case3_bool():
    assert bool_bench_case(map_case3).verify_convert() == 0


def test_case2_single_bit():
    assert length1_bench_case(map_case3).verify_convert() == 0


def test_case3_single_bit():
    assert length1_bench_case(map_case3).verify_convert() == 0



import os

from myhdl import Cosimulation

cmd = "iverilog -o inc.o -Dn=%s " + \
      "../../test/verilog/inc.v " + \
      "../../test/verilog/dut_inc.v "
      
def inc(count, enable, clock, reset, n):
    os.system(cmd % n)
    return Cosimulation("vvp -m ../myhdl.vpi inc.o", **locals())
               



from myhdl import Signal, ResetSignal, modbv

from inc import inc

def convert_inc(hdl):
    """Convert inc block to Verilog or VHDL."""

    m = 8

    count = Signal(modbv(0)[m:])
    enable = Signal(bool(0))
    clock  = Signal(bool(0))
    reset = ResetSignal(0, active=0, isasync=True)

    inc_1 = inc(count, enable, clock, reset)

    inc_1.convert(hdl=hdl)


convert_inc(hdl='Verilog')
convert_inc(hdl='VHDL')



import os
path = os.path

from random import seed, shuffle
seed('We want repeatable randomness')

from myhdl import (block, Signal, modbv, intbv, delay, instance, conversion)

N = 8
M = 2 ** N
DEPTH = 5


@block
def xor(z, a, b, c):

    @instance
    def comb():
        while 1:
            yield a, b, c
            z.next = a ^ b ^ c

    return comb


def randOthers(i, n):
    l = list(range(n))
    l.remove(i)
    shuffle(l)
    return l[0], l[1]


@block
def randscrambler(ol, il, stage=0):
    """ Recursive hierarchy of random xor gates.

    An invented module to check hierarchy with toVerilog.

    """

    sl1 = [Signal(bool()) for __ in range(N)]
    sl2 = [Signal(bool()) for __ in range(N)]
    i1 = [None] * N
    i2 = [None] * N

    if stage < DEPTH:
        for i in range(N):
            j, k = randOthers(i, N)
            i1[i] = xor(sl1[i], il[i], il[j], il[k])
        rs = randscrambler(sl2, sl1, stage=stage + 1)
        for i in range(N):
            j, k = randOthers(i, N)
            i2[i] = xor(ol[i], sl2[i], sl2[j], sl2[k])
        return i1, i2, rs
    else:
        for i in range(N):
            j, k = randOthers(i, N)
            i1[i] = xor(ol[i], il[i], il[j], il[k])
        return i1


@block
def randscrambler_top(o7, o6, o5, o4, o3, o2, o1, o0,
                    i7, i6, i5, i4, i3, i2, i1, i0):
    sl1 = [i7, i6, i5, i4, i3, i2, i1, i0]
    sl2 = [o7, o6, o5, o4, o3, o2, o1, o0]
    rs = randscrambler(sl2, sl1, stage=0)
    return rs


o7, o6, o5, o4, o3, o2, o1, o0 = [Signal(bool()) for i in range(N)]
i7, i6, i5, i4, i3, i2, i1, i0 = [Signal(bool()) for i in range(N)]
v7, v6, v5, v4, v3, v2, v1, v0 = [Signal(bool()) for i in range(N)]


@block
def randscramblerBench():

    @instance
    def stimulus():
        a = modbv(0)[N:]
        z = intbv(0)[N:]
        for i in range(100):

            a[:] += 97
            i7.next = a[7]
            i6.next = a[6]
            i5.next = a[5]
            i4.next = a[4]
            i3.next = a[3]
            i2.next = a[2]
            i1.next = a[1]
            i0.next = a[0]
            yield delay(10)
            z[7] = o7
            z[6] = o6
            z[5] = o5
            z[4] = o4
            z[3] = o3
            z[2] = o2
            z[1] = o1
            z[0] = o0
            print (a)
            print (z)

    rs = randscrambler_top(
        o7, o6, o5, o4, o3, o2, o1, o0,
        i7, i6, i5, i4, i3, i2, i1, i0
    )

    return rs, stimulus


def test_randscramber():
    assert conversion.verify(randscramblerBench()) == 0



import os
path = os.path

from myhdl import (block, Signal, intbv, delay, always_comb,
                   always, instance, StopSimulation,
                   conversion, toVHDL
                   )


@block
def ternary1(dout, clk, rst):

    @always(clk.posedge, rst.negedge)
    def comb():
        if rst == 0:
            dout.next = 0
        else:
            dout.next = (dout + 1) if dout < 127 else 0

    return comb


@block
def ternary2(dout, clk, rst):

    dout_d = Signal(intbv(0)[len(dout):])

    @always(clk.posedge, rst.negedge)
    def synch():
        if rst == 0:
            dout.next = 0
        else:
            dout.next = dout_d

    @always_comb
    def comb():
        dout_d.next = (dout + 1) if dout < 127 else 0

    return synch, comb


@block
def TernaryBench(ternary):

    dout = Signal(intbv(0)[8:])
    clk = Signal(bool(0))
    rst = Signal(bool(0))

    ternary_inst = ternary(dout, clk, rst)

    @instance
    def stimulus():
        rst.next = 1
        clk.next = 0
        yield delay(10)
        rst.next = 0
        yield delay(10)
        rst.next = 1
        yield delay(10)
        for i in range(1000):
            clk.next = 1
            yield delay(10)
            assert dout == (i + 1) % 128
            print(dout)
            clk.next = 0
            yield delay(10)

        raise StopSimulation()

    return stimulus, ternary_inst


# uncomment when we have a VHDL-2008 compliant simulator
def test_ternary1():
    toVHDL.name = 'ternary1'
    assert conversion.verify(TernaryBench(ternary1)) == 0


def test_ternary2():
    toVHDL.name = 'ternary2'
    assert conversion.verify(TernaryBench(ternary2)) == 0




import os
path = os.path
import unittest
from unittest import TestCase
import pytest
import random
from random import randrange
random.seed(2)

from myhdl import (block, Signal, intbv, delay, always, always_comb,
                   instance, StopSimulation)
from myhdl._Simulation import Simulation

from .util import setupCosimulation

from myhdl import ConversionError
from myhdl.conversion._misc import _error

ACTIVE_LOW, INACTIVE_HIGH = 0, 1


@block
def incRef(count, enable, clock, reset, n):
    """ Incrementer with enable.
    
    count -- output
    enable -- control input, increment when 1
    clock -- clock input
    reset -- asynchronous reset input
    n -- counter max value
    """

    @instance
    def comb():
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    count.next = (count + 1) % n

    return comb


@block
def incGen(count, enable, clock, reset, n):
    """ Generator with __verilog__ is not permitted """

    @instance
    def comb():
        incGen.verilog_code = "Template string"
        while 1:
                yield clock.posedge, reset.negedge
                if reset == ACTIVE_LOW:
                    count.next = 0
                else:
                    if enable:
                        count.next = (count + 1) % n

    return comb


@block
def inc(count, enable, clock, reset, n):
    """ Incrementer with enable.
    
    count -- output
    enable -- control input, increment when 1
    clock -- clock input
    reset -- asynchronous reset input
    n -- counter max value
    """

    @always(clock.posedge, reset.negedge)
    def incProcess():
        # make it fail in conversion
        import types
        if reset == ACTIVE_LOW:
            count.next = 0
        else:
            if enable:
                count.next = (count + 1) % n

    count.driven = "reg"

    inc.verilog_code = \
"""
always @(posedge $clock, negedge $reset) begin
    if (reset == 0) begin
        $count <= 0;
    end
    else begin
        if (enable) begin
            $count <= ($count + 1) % $n;
        end
    end
end
"""

    return incProcess


@block
def incErr(count, enable, clock, reset, n):

    @always(clock.posedge, reset.negedge)
    def incProcess():
        # make it fail in conversion
        import types
        if reset == ACTIVE_LOW:
            count.next = 0
        else:
            if enable:
                count.next = (count + 1) % n

    count.driven = "reg"

    incErr.verilog_code = \
"""
always @(posedge $clock, negedge $reset) begin
    if (reset == 0) begin
        $count <= 0;
    end
    else begin
        if (enable) begin
            $count <= ($countq + 1) % $n;
        end
    end
end
"""

    return incProcess


@block
def inc_comb(nextCount, count, n):

    @always_comb
    def comb():
        # make if fail in conversion
        import types
        nextCount.next = (count + 1) % n

    nextCount.driven = "wire"

    inc_comb.verilog_code = \
"""
assign $nextCount = ($count + 1) % $n;
"""

    return comb


@block
def inc_seq(count, nextCount, enable, clock, reset):

    @always(clock.posedge, reset.negedge)
    def comb():
        if reset == ACTIVE_LOW:
            count.next = 0
        else:
            if (enable):
                count.next = nextCount

    count.driven = "reg"

    inc_seq.verilog_code = \
"""
always @(posedge $clock, negedge $reset) begin
    if (reset == 0) begin
        $count <= 0;
    end
    else begin
        if (enable) begin
            $count <= $nextCount;
        end
    end
end
"""
    # return nothing - cannot be simulated
    return []


@block
def inc2(count, enable, clock, reset, n):

    nextCount = Signal(intbv(0, min=0, max=n))

    comb = inc_comb(nextCount, count, n)
    seq = inc_seq(count, nextCount, enable, clock, reset)

    return comb, seq


@block
def inc3(count, enable, clock, reset, n):
    inc2_inst = inc2(count, enable, clock, reset, n)
    return inc2_inst


@block
def inc_v(name, count, enable, clock, reset):
    return setupCosimulation(**locals())


class TestInc(TestCase):

    def clockGen(self, clock):
        while 1:
            yield delay(10)
            clock.next = not clock

    def stimulus(self, enable, clock, reset):
        reset.next = INACTIVE_HIGH
        yield clock.negedge
        reset.next = ACTIVE_LOW
        yield clock.negedge
        reset.next = INACTIVE_HIGH
        for dummy in range(1000):
            enable.next = 1
            yield clock.negedge
        for dummy in range(1000):
            enable.next = min(1, randrange(5))
            yield clock.negedge
        raise StopSimulation

    def check(self, count, count_v, enable, clock, reset, n):
        expect = 0
        yield reset.posedge
        self.assertEqual(count, expect)
        self.assertEqual(count, count_v)
        while 1:
            yield clock.posedge
            if enable:
                expect = (expect + 1) % n
            yield delay(1)
            # print "%d count %s expect %s count_v %s" % (now(), count, expect, count_v)
            self.assertEqual(count, expect)
            self.assertEqual(count, count_v)

    def bench(self, incRef, incVer):

        m = 8
        n = 2 ** m

        count = Signal(intbv(0)[m:])
        count_v = Signal(intbv(0)[m:])
        enable = Signal(bool(0))
        clock, reset = [Signal(bool()) for __ in range(2)]

        inc_inst_ref = incRef(count, enable, clock, reset, n=n)
        inc_inst = incVer(count, enable, clock, reset, n=n).convert(hdl='Verilog')
        # inc_inst = inc(count, enable, clock, reset, n=n)
        inc_inst_v = inc_v(incVer.__name__, count_v, enable, clock, reset)
        clk_1 = self.clockGen(clock)
        st_1 = self.stimulus(enable, clock, reset)
        ch_1 = self.check(count, count_v, enable, clock, reset, n=n)

        sim = Simulation(inc_inst_ref, inc_inst_v, clk_1, st_1, ch_1)
        return sim

    def testIncRefIncRef(self):
        """ Check increment operation """
        sim = self.bench(incRef, incRef)
        sim.run(quiet=1)

    def testIncRefInc(self):
        sim = self.bench(incRef, inc)
        sim.run(quiet=1)

    def testIncInc(self):
        sim = self.bench(inc, inc)
        sim.run(quiet=1)

    @pytest.mark.xfail
    def testIncRefInc2(self):
        sim = self.bench(incRef, inc2)
        sim.run(quiet=1)

    @pytest.mark.xfail
    def testIncRefInc3(self):
        sim = self.bench(incRef, inc3)
        sim.run(quiet=1)

    def testIncGen(self):
        m = 8
        n = 2 ** m
        count_v = Signal(intbv(0)[m:])
        enable = Signal(bool(0))
        clock, reset = [Signal(bool()) for __ in range(2)]
        try:
            inc_inst = incGen(count_v, enable, clock, reset, n=n).convert(hdl='Verilog')
        except ConversionError as e:
            self.assertEqual(e.kind, _error.NotSupported)
        else:
            self.fail()

    def testIncErr(self):
        m = 8
        n = 2 ** m
        count_v = Signal(intbv(0)[m:])
        enable = Signal(bool(0))
        clock, reset = [Signal(bool()) for _ in range(2)]
        try:
            inc_inst = incErr(count_v, enable, clock, reset, n=n).convert(hdl='Verilog')
        except ConversionError:
            pass
        else:
            self.fail()


if __name__ == '__main__':
    unittest.main()




import os
path = os.path
import unittest
from random import randrange

from myhdl import (block, Signal, intbv, delay,
                   instance, downrange)
from myhdl._Simulation import Simulation

from .util import setupCosimulation


@block
def ForLoop1(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in downrange(len(a)):
                if a[i] == 1:
                    var += 1
            out.next = var

    return comb


@block
def ForLoop2(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in downrange(len(a), 5):
                if a[i] == 1:
                    var += 1
            out.next = var

    return comb


@block
def ForLoop3(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in downrange(len(a), 3, 2):
                if a[i] == 1:
                    var += 1
            out.next = var

    return comb


@block
def ForLoop4(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in range(len(a)):
                if a[i] == 1:
                    var += 1
            out.next = var

    return comb


@block
def ForLoop5(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in range(6, len(a)):
                if a[i] == 1:
                    var += 1
            out.next = var

    return comb


@block
def ForLoop6(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in range(5, len(a), 3):
                if a[i] == 1:
                    var += 1
            out.next = var

    return comb


@block
def ForContinueLoop(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in downrange(len(a)):
                if a[i] == 0:
                    continue
                var += 1
            out.next = var

    return comb


@block
def ForBreakLoop(a, out):

    @instance
    def comb():
        while 1:
            yield a
            out.next = 0
            for i in downrange(len(a)):
                if a[i] == 1:
                    out.next = i
                    break

    return comb


@block
def ForBreakContinueLoop(a, out):

    @instance
    def comb():
        while 1:
            yield a
            out.next = 0
            for i in downrange(len(a)):
                if a[i] == 0:
                    continue
                out.next = i
                break

    return comb


@block
def NestedForLoop1(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in downrange(len(a)):
                if a[i] == 0:
                    continue
                else:
                    for j in downrange(i):
                        if a[j] == 0:
                            var += 1
                    break
            out.next = var

    return comb


@block
def NestedForLoop2(a, out):

    @instance
    def comb():
        while 1:
            yield a
            out.next = 0
            for i in downrange(len(a)):
                if a[i] == 0:
                    continue
                else:
                    for j in downrange(i - 1):
                        if a[j] == 0:
                            pass
                        else:
                            out.next = j
                            break
                    break

    return comb


def ReturnFromFunction(a):
    for i in downrange(len(a)):
        if a[i] == 1:
            return i
    return 0


@block
def FunctionCall(a, out):

    @instance
    def comb():
        while 1:
            yield a
            out.next = ReturnFromFunction(a)

    return comb


# During the following check, I noticed that non-blocking assignments
# are not scheduled when a task is disabled in Icarus. Apparently
# this is one of the many vague areas in the Verilog standard.
def ReturnFromTask(a, out):
    for i in downrange(len(a)):
        if a[i] == 1:
            out[:] = i
            return
    out[:] = 23  # to notice it


@block
def TaskCall(a, out):

    @instance
    def comb():
        var = intbv(0)[8:]
        while 1:
            yield a
            ReturnFromTask(a, var)
            out.next = var

    return comb


@block
def WhileLoop(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            i = len(a) - 1
            while i >= 0:
                if a[i] == 1:
                    var += 1
                i -= 1
            out.next = var

    return comb


@block
def WhileContinueLoop(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            i = len(a) - 1
            while i >= 0:
                if a[i] == 0:
                    i -= 1
                    continue
                var += 1
                i -= 1
            out.next = var

    return comb


@block
def WhileBreakLoop(a, out):

    @instance
    def comb():
        while 1:
            yield a
            i = len(a) - 1
            out.next = 0
            while i >= 0:
                if a[i] == 1:
                    out.next = i
                    break
                i -= 1

    return comb


@block
def WhileBreakContinueLoop(a, out):

    @instance
    def comb():
        while 1:
            yield a
            i = len(a) - 1
            out.next = 0
            while i >= 0:
                if a[i] == 0:
                    i -= 1
                    continue
                out.next = i
                break

    return comb


@block
def LoopTest_v(name, a, out):
    return setupCosimulation(**locals())


class TestLoops(unittest.TestCase):

    def bench(self, LoopTest):

        a = Signal(intbv(-1)[16:])
        out_v = Signal(intbv(0)[16:])
        out = Signal(intbv(0)[16:])

        looptest_inst = LoopTest(a, out).convert(hdl='Verilog')
        # looptest_inst = LoopTest(hec, header)
        looptest_v_inst = LoopTest_v(LoopTest.__name__, a, out_v)

        def stimulus():
            for i in range(100):
                a.next = randrange(2 ** min(i, 16))
                yield delay(10)
                # print "%s %s" % (out, out_v)
                self.assertEqual(out, out_v)

        return stimulus(), looptest_inst, looptest_v_inst

    def testForLoop1(self):
        sim = self.bench(ForLoop1)
        Simulation(sim).run()

    def testForLoop2(self):
        sim = self.bench(ForLoop2)
        Simulation(sim).run()

    def testForLoop3(self):
        sim = self.bench(ForLoop3)
        Simulation(sim).run()

    def testForLoop4(self):
        sim = self.bench(ForLoop4)
        Simulation(sim).run()

    def testForLoop5(self):
        sim = self.bench(ForLoop5)
        Simulation(sim).run()

    def testForLoop6(self):
        sim = self.bench(ForLoop6)
        Simulation(sim).run()

    def testForContinueLoop(self):
        sim = self.bench(ForContinueLoop)
        Simulation(sim).run()

    def testForBreakLoop(self):
        sim = self.bench(ForBreakLoop)
        Simulation(sim).run()

    def testForBreakContinueLoop(self):
        sim = self.bench(ForBreakContinueLoop)
        Simulation(sim).run()

    def testNestedForLoop1(self):
        sim = self.bench(NestedForLoop1)
        Simulation(sim).run()

    def testNestedForLoop2(self):
        sim = self.bench(NestedForLoop2)
        Simulation(sim).run()

    def testFunctionCall(self):
        sim = self.bench(FunctionCall)
        Simulation(sim).run()

    def testTaskCall(self):
        sim = self.bench(TaskCall)
        Simulation(sim).run()

    def testWhileLoop(self):
        sim = self.bench(WhileLoop)
        Simulation(sim).run()

    def testWhileContinueLoop(self):
        sim = self.bench(WhileContinueLoop)
        Simulation(sim).run()

    def testWhileBreakLoop(self):
        sim = self.bench(WhileBreakLoop)
        Simulation(sim).run()

    def testWhileBreakContinueLoop(self):
        sim = self.bench(WhileBreakContinueLoop)
        Simulation(sim).run()


if __name__ == '__main__':
    unittest.main()



import os

from tempfile import mkdtemp
from shutil import rmtree

from myhdl import (block, Signal, intbv, always, toVHDL, toVerilog)

from myhdl import __version__
_version = __version__.replace('.', '')
_shortversion = _version.replace('dev', '')[:-2]


@block
def simple_dir_model(din, dout, clk):
    """ Simple convertible model """

    @always(clk.posedge)
    def register():
            dout.next = din

    return register


def test_toVHDL_set_dir():
    '''In order that a developer can define where in the project 
    hierarchy any generated VHDL files should be placed, it should be 
    possible to set a directory attribute on toVHDL controlling this.
    '''

    tmp_dir = mkdtemp()

    din = Signal(intbv(0)[5:])
    dout = Signal(intbv(0)[5:])
    clock = Signal(bool(0))

    try:
        toVHDL.directory = tmp_dir
        toVHDL(simple_dir_model(din, dout, clock))

        assert os.path.exists(os.path.join(tmp_dir, 'simple_dir_model.vhd'))

    finally:
        toVHDL.directory = None
        rmtree(tmp_dir)


def test_toVHDL_myhdl_package_set_dir():
    '''In order that the MyHDL package files are located in the 
    same place as the generated VHDL files, when the directory attribute of 
    toVHDL is set, this location should be used for the generated MyHDL 
    package files.
    '''
    tmp_dir = mkdtemp()

    din = Signal(intbv(0)[5:])
    dout = Signal(intbv(0)[5:])
    clock = Signal(bool(0))

    try:
        toVHDL.directory = tmp_dir
        toVHDL(simple_dir_model(din, dout, clock))

        assert os.path.exists(
            os.path.join(tmp_dir, "pck_myhdl_%s.vhd" % _shortversion))

    finally:

        toVHDL.directory = None

        rmtree(tmp_dir)


def test_toVerilog_set_dir():
    '''In order that a developer can define where in the project 
    hierarchy any generated Verilog files should be placed, it should be 
    possible to set a directory attribute on toVerilog controlling this.
    '''

    tmp_dir = mkdtemp()

    din = Signal(intbv(0)[5:])
    dout = Signal(intbv(0)[5:])
    clock = Signal(bool(0))

    no_testbench_state = toVerilog.no_testbench
    toVerilog.no_testbench = True

    try:
        toVerilog.directory = tmp_dir
        toVerilog(simple_dir_model(din, dout, clock))

        assert os.path.exists(os.path.join(tmp_dir, 'simple_dir_model.v'))

    finally:
        toVerilog.directory = None
        toVerilog.no_testbench = no_testbench_state
        rmtree(tmp_dir)


def test_toVerilog_testbench_set_dir():
    '''In order that generated Verilog test bench files are located in the 
    same place as the Verilog files, when the directory attribute of 
    toVerilog is set, this location should be used for the generated test
    bench files.
    '''

    tmp_dir = mkdtemp()

    din = Signal(intbv(0)[5:])
    dout = Signal(intbv(0)[5:])
    clock = Signal(bool(0))

    no_testbench_state = toVerilog.no_testbench
    toVerilog.no_testbench = False

    try:
        toVerilog.directory = tmp_dir
        toVerilog(simple_dir_model(din, dout, clock))

        assert os.path.exists(os.path.join(tmp_dir, 'tb_simple_dir_model.v'))

    finally:

        toVerilog.directory = None
        toVerilog.no_testbench = no_testbench_state

        rmtree(tmp_dir)



from myhdl import (block, Signal, intbv, modbv, delay, always_seq,
                   ResetSignal, instance, instances, StopSimulation,
                   # conversion)
                   )


@block
def chunk_buffer(Clk, Reset, Input, Output):
    OUTPUT_BIT_COUNT = len(Output)
    idx = Signal(modbv(0)[(len(Input) // OUTPUT_BIT_COUNT):])

    @always_seq(Clk.posedge, reset=Reset)
    def goto_next():
        idx.next = idx + 1
        Output.next = Input[idx + OUTPUT_BIT_COUNT:idx]

    return instances()


@block
def chunk_buffer_sim(Clk, Reset, Input, Output):

    dut = chunk_buffer(Clk, Reset, Input, Output)

    tCK = 20
    tReset = int(tCK * 3.5)

    @instance
    def tb_clk():
        Clk.next = False
        yield delay(int(tCK // 2))
        while True:
            Clk.next = not Clk
            yield delay(int(tCK // 2))

    @instance
    def tb_logic():
        Reset.next = 1
        yield delay(tReset)
        Reset.next = 0

        Input.next = 0xABCDEF

        for dummy in range(100):
            yield delay(int(tCK // 2))
        raise StopSimulation

    return instances()


def test_top_level_interfaces_verify():
    Clk = Signal(bool(0))
    Reset = ResetSignal(0, 1, True)
    Input = Signal(intbv(0)[256:])
    Output = Signal(intbv(0)[8:])

    top_sim = chunk_buffer_sim(Clk, Reset, Input, Output)
    # assert conversion.analyze(top_sim) == 0
    # as top_sim is  a Block object, let's use the proper method
    assert top_sim.analyze_convert() == 0

    top = chunk_buffer(Clk, Reset, Input, Output)
    top.name = 'ChunkBuffer'
    top.convert('Verilog')
    top.convert('VHDL')



#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Run the unit tests for Signal """
import random

from myhdl import instance, instances

random.seed(1)  # random, but deterministic


def A(n):

    @instance
    def comb():
        yield None

    return comb


def B(n):

    @instance
    def comb():
        yield None

    return comb


def C(n):
    A_1 = A(1)
    A_2 = A(2)
    B_1 = B(1)
    return A_1, A_2, B_1


g = 3


class TestInstances:

    def testInstances(self):

        @instance
        def D_1():
            yield None

        d = 1

        A_1 = A(1)
        a = [1, 2]
        B_1 = B(1)
        b = "string"
        C_1 = C(1)
        c = {}

        i = instances()
        # can't just construct an expected list;
        # that would become part of the instances also!
        assert len(i) == 4
        for e in (D_1, A_1, B_1, C_1):
            assert e in i



import myhdl
from myhdl import *

from test_lfsr24 import test_lfsr24
from test_randgen import test_randgen
from test_longdiv import test_longdiv
from test_timer import test_timer
from timer import timer_sig, timer_var
from test_findmax import test_findmax

toVerilog(test_lfsr24)
toVHDL(test_lfsr24)

toVerilog(test_randgen)
toVHDL(test_randgen)

toVerilog(test_longdiv)
toVHDL(test_longdiv)

toVerilog(test_timer, timer_var)
toVHDL(test_timer, timer_var)

toVerilog(test_findmax)
toVHDL(test_findmax)



from myhdl import (block, Signal, intbv, always_seq, instances, ResetSignal)


@block
def mpegChannel(clk, rst):

    s_tx_data_xor_mask_r = Signal(intbv(0)[1 + 31:])

    @always_seq(clk.posedge, rst)
    def fsm_seq():
        for i in range(4):
            if i == 0:
                s_tx_data_xor_mask_r.next[1 + 7:0] = 0
            elif i == 1:
                s_tx_data_xor_mask_r.next[1 + 15:8] = 1
            elif i == 2:
                s_tx_data_xor_mask_r.next[1 + 23:16] = 2
            else:
                s_tx_data_xor_mask_r.next[1 + 31:24] = 3

    return instances()


def test_issue_40():
    clk = Signal(bool(0))
    rst = ResetSignal(0, active=1, isasync=True)

    assert mpegChannel(clk, rst).analyze_convert() == 0




import os
path = os.path

from myhdl import (block, Signal, intbv, delay, always_seq,
                   ResetSignal, instance, StopSimulation
                   )


@block
def NonlocalBench():

    ALL_ONES = 2 ** 7 - 1
    ONE = 1

    qout = Signal(intbv(ONE)[8:])
    init = Signal(bool(0))
    clk = Signal(bool(0))
    reset = ResetSignal(0, active=1, isasync=True)

    q = intbv(ONE)[8:]

    @always_seq(clk.posedge, reset=reset)
    def scrambler():
        if init:
            q[8:1] = ALL_ONES
        else:
            q[0] = q[7] ^ q[6]
            q[8:1] = q[7:0]
        qout.next = q[8:1]

    @instance
    def clkgen():
        clk.next = 1
        while True:
            yield delay(10)
            clk.next = not clk

    @instance
    def stimulus():
        reset.next = 0
        init.next = 0
        yield clk.negedge
        reset.next = 1
        yield clk.negedge
        print(qout)
        assert qout == ONE
        reset.next = 0
        for dummy in range(100):
            yield clk.negedge
            print(qout)
        init.next = 1
        yield clk.negedge
        assert qout == ALL_ONES
        print(qout)
        init.next = 0
        for dummy in range(300):
            print(qout)
        raise StopSimulation()

    return scrambler, clkgen, stimulus


def test_nonlocal():
    assert NonlocalBench().verify_convert() == 0




from myhdl import (block, Signal, enum, intbv, delay, instance)
from myhdl import ConversionError
from myhdl.conversion._misc import _error

t_State = enum("START", "RUN", "STOP")


@block
def PrintBench():
    si1 = Signal(intbv(0)[8:])
    si2 = Signal(intbv(0, min=-10, max=12))
    sb = Signal(bool(0))

    @instance
    def comb():
        i1 = intbv(0)[8:]
        i2 = intbv(0, min=-10, max=12)
        b = bool(1)
        state = t_State.START
        i1[:] = 10
        si1.next = 11
        i2[:] = -7
        si2.next = -5
        yield delay(10)
        print('')
        print(i1)
        print(i2)
        print("%d %d" % (i1, i2))
        print(si1)
        print(si2)

        yield delay(10)
        print("This is a test")

        yield delay(10)
        print(int(b))
        print(int(sb))

        yield delay(10)
        print("i1 is %s" % i1)

        yield delay(10)
        print("i1 is %s, i2 is %s" % (i1, i2))
        print("i1 %s i2 %s b %s si1 %s si2 %s" % (i1, i2, b, si1, si2))
        print("i1 %d i2 %d b %d si1 %d si2 %d" % (i1, i2, b, si1, si2))
        print(b)
        # print "%% %s" % i1

        yield delay(10)
        print(state)
        print("the state is %s" % state)
        print("the state is %s" % (state,))
        print("i1 is %s and the state is %s" % (i1, state))

        # ord test
        yield delay(10)
        print(ord('y'))
        print(ord('2'))

        # signed
        yield delay(10)
        print(i1.signed())
        print(i2.signed())
        print(si1.signed())
        print(si2.signed())

    return comb


def testPrint():
    assert PrintBench().verify_convert() == 0


@block
def PrintLongVectorsBench():
    N84 = 84
    M84 = 2 ** N84 - 1
    N85 = 85
    M85 = 2 ** N85 - 1
    N86 = 86
    M86 = 2 ** N86 - 1
    N87 = 87

    si1 = Signal(intbv(0)[N87:])
    si2 = Signal(intbv(0, min=-M85, max=M86))

    @instance
    def comb():
        i1 = intbv(0)[N85:]
        i2 = intbv(0, min=-M86, max=M85)
        si1.next = 0
        si2.next = 0
        yield delay(10)
        print("%s %s %s %s" % (i1, i2, si1, si2))
        i1[:] = M84
        i2[:] = -1
        si1.next = M85
        si2.next = -1
        yield delay(10)
        print("%s %s %s %s" % (i1, i2, si1, si2))
        i1[:] = 596854
        i2[:] = -4954502
        si1.next = 232434
        si2.next = -3434355
        yield delay(10)
        print("%s %s %s %s" % (i1, i2, si1, si2))

    return comb


def testPrintLongVectors():
    assert PrintLongVectorsBench().verify_convert() == 0

# format string errors and unsupported features


@block
def PrintError1():

    @instance
    def comb():
        i1 = intbv(12)[8:]
        yield delay(10)
        print("floating point %f end" % i1)

    return comb


def testPrintError1():
    try:
        PrintError1().verify_convert()
    except ConversionError as e:
        assert e.kind == _error.UnsupportedFormatString
    else:
        assert False


@block
def PrintError2():

    @instance
    def comb():
        i1 = intbv(12)[8:]
        yield delay(10)
        print("begin %s %s end" % i1)

    return comb


def testPrintError2():
    try:
        PrintError2().verify_convert()
    except ConversionError as e:
        assert e.kind == _error.FormatString
    else:
        assert False


@block
def PrintError3():

    @instance
    def comb():
        i1 = intbv(12)[8:]
        i2 = intbv(13)[8:]
        yield delay(10)
        print("begin %s end" % (i1, i2))

    return comb


def testPrintError3():
    try:
        PrintError3().verify_convert()
    except ConversionError as e:
        assert e.kind == _error.FormatString
    else:
        assert False


@block
def PrintError4():

    @instance
    def comb():
        i1 = intbv(12)[8:]
        yield delay(10)
        print("%10s" % i1)

    return comb


def testPrintError4():
    try:
        PrintError4().verify_convert()
    except ConversionError as e:
        assert e.kind == _error.UnsupportedFormatString
    else:
        assert False


@block
def PrintError5():

    @instance
    def comb():
        i1 = intbv(12)[8:]
        yield delay(10)
        print("%-10s" % i1)

    return comb


def testPrintError5():
    try:
        PrintError5().verify_convert()
    except ConversionError as e:
        assert e.kind == _error.UnsupportedFormatString
    else:
        assert False



from math import pi, sin, cos, log
import random

import myhdl
from myhdl import *

from SineComputer import SineComputer, SineComputer_v


def bench(fractionSize, errorMargin, nrTests=100):
    
    """ Test bench for SineComputer.

    fractionSize: number of bits after the point
    errorMargin: margin for rounding errors on result
    nrTests: number of tests vectors

    """

    # scaling factor to represent floats as integers
    M = 2**fractionSize

    # maximum angle
    ZMAX = int(round(M*pi/2))
    
    # error margin shorthand
    D = errorMargin

    # signals
    cos_z0 = Signal(intbv(0, min=-D, max=M+D))
    sin_z0 = Signal(intbv(0, min=-M-D, max=M+D))
    z0 = Signal(intbv(0, min=-ZMAX, max=ZMAX+1))
    done = Signal(False)
    start = Signal(False)
    clock = Signal(bool(0))
    reset = Signal(True)

    # design under test
    # dut = SineComputer(cos_z0, sin_z0, done, z0, start, clock, reset)
    dut = SineComputer_v(cos_z0, sin_z0, done, z0, start, clock, reset)

    # clock generator
    @always(delay(10))
    def clockgen():
        clock.next = not clock

    # test vector setup
    testAngles = [-pi/2, -pi/4, 0.0, pi/4, pi/2]
    testAngles.extend([random.uniform(-pi/2, pi/2) for i in range(nrTests)])
##     testAngles.extend([random.uniform(-0.01, 0.01) for i in range(nrTests)])
##     testAngles.extend([random.uniform(pi/2-0.01, pi/2) for i in range(nrTests)])
##     testAngles.extend([random.uniform(-pi/2, -pi/2+0.01) for i in range(nrTests)])
    
    # actual test 
    @instance
    def check():
        yield clock.negedge
        reset.next = False
        for z in testAngles:
            yield clock.negedge
            z0.next = int(round(M*z))
            start.next = True
            yield clock.negedge
            start.next = False
            yield done.posedge
            exp_cos_z0 = int(round(cos(z)*M))
            exp_sin_z0 = int(round(sin(z)*M))
            assert abs(cos_z0 - exp_cos_z0) < D
            assert abs(sin_z0 - exp_sin_z0) < D

        raise StopSimulation

    return dut, clockgen, check

def test_bench():
    fractionSize = 18
    errorMargin = fractionSize
    tb = bench(fractionSize, errorMargin)
    sim = Simulation(tb)
    sim.run()

if __name__ == '__main__':
    test_bench()



import myhdl
from myhdl import *

ACTIVE = 0
DirType = enum('RIGHT', 'LEFT')


def jc2(goLeft, goRight, stop, clk, q):

    """ A bi-directional 4-bit Johnson counter with stop control.

    I/O pins:
    --------
    clk      : input free-running slow clock 
    goLeft   : input signal to shift left (active-low switch)
    goRight   : input signal to shift right (active-low switch)
    stop     : input signal to stop counting (active-low switch)
    q        : 4-bit counter output (active-low LEDs; q[0] is right-most)

    Operation:
    ---------
    The counter is triggered on the rising edge of the clock (clk). 
    A low pulse on the goLeft input will cause the counter to start 
    shifting left from its current state. A low pulse on the goRight
    input will cause the counter to start shifting right from its 
    current state. A low pulse on the stop input will cause the 
    counter to hold its current state until goLeft or goRight is pulsed.

    After power-up, the counter is stopped with all outputs low (LEDs lit).

    """

    dir = Signal(DirType.LEFT)
    run = Signal(False)

    @always(clk.posedge)
    def synch():
        # direction
        if goRight == ACTIVE:
            dir.next = DirType.RIGHT
            run.next = True
        elif goLeft == ACTIVE:
            dir.next = DirType.LEFT
            run.next = True
        # stop
        if stop == ACTIVE:
            run.next = False
        # counter action
        if run:
            if dir == DirType.LEFT:
                q.next[4:1] = q[3:]
                q.next[0] = not q[3]
            else:
                q.next[3:] = q[4:1]
                q.next[3] = not q[0]

    return synch



from myhdl import (block, Signal, enum, intbv, delay, always_comb,
                   always, instance, instances, StopSimulation,
                   concat
                   )

# SELOPTS = enum('SEL0', 'SEL1', 'SEL2', 'SEL3')


@block
def mux4a(
    sel,
    in0,
    in1,
    in2,
    in3,
    out0
):

    @always_comb
    def rtl():
        if sel == 0:
            out0.next = in0
        elif sel == 1:
            out0.next = in1
        elif sel == 2:
            out0.next = in2
        else:
            out0.next = in3

    return instances()


@block
def mux4b(sel, in0, in1, in2, in3, out0):

    @always_comb
    def rtl():
        match sel:
            case 0:
                out0.next = in0
            case 1:
                out0.next = in1
            case 2:
                out0.next = in2
            case _:
                out0.next = in3

    return instances()


@block
def mux4c(sel, in0, in1, in3, out0):

    @always_comb
    def rtl():
        match sel:
            case 0:
                out0.next = in0
            case 1 | 2:
                out0.next = in1
            case _:
                out0.next = in3

    return instances()


t_opts = enum('SEL0', 'SEL1', 'SEL2', 'SEL3')


@block
def enumMux4a(
    sel,
    in0,
    in1,
    in2,
    in3,
    out0,
):

    sel_enum = Signal(t_opts.SEL0)

    @always_comb
    def mapping():
        if 0 == sel:
            sel_enum.next = t_opts.SEL0
        elif 1 == sel:
            sel_enum.next = t_opts.SEL1
        elif 2 == sel:
            sel_enum.next = t_opts.SEL2
        elif 3 == sel:
            sel_enum.next = t_opts.SEL3

    @always_comb
    def rtl():
        if sel_enum == t_opts.SEL0:
            out0.next = in0
        elif sel_enum == t_opts.SEL1:
            out0.next = in1
        elif sel_enum == t_opts.SEL2:
            out0.next = in2
        elif sel_enum == t_opts.SEL3:
            out0.next = in3

    return instances()


@block
def enumMux4b(sel, in0, in1, in2, in3, out0):
    sel_enum = Signal(t_opts.SEL0)

    @always_comb
    def mapping():
        if 0 == sel:
            sel_enum.next = t_opts.SEL0
        elif 1 == sel:
            sel_enum.next = t_opts.SEL1
        elif 2 == sel:
            sel_enum.next = t_opts.SEL2
        elif 3 == sel:
            sel_enum.next = t_opts.SEL3

    @always_comb
    def rtl():
        match sel_enum:
            case t_opts.SEL0:
                out0.next = in0
            case t_opts.SEL1:
                out0.next = in1
            case t_opts.SEL2:
                out0.next = in2
            case _:
                out0.next = in3

    return instances()


t_fsma_opts = enum('IDLE', 'READ', 'WRITE', 'ERROR')


@block
def fsm4a(
    clk,
    rst,
    rd,
    wr,
):
    smp = Signal(t_fsma_opts.IDLE)

    @always(clk.posedge)
    def rtl():

        if rst:
            smp.next = t_fsma_opts.IDLE
        else:
            match smp:
                case t_fsma_opts.IDLE:
                    smp.next = t_fsma_opts.IDLE
                    if rd:
                        smp.next = t_fsma_opts.READ
                    if wr:
                        smp.next = t_fsma_opts.WRITE
                case t_fsma_opts.READ:
                    smp.next = t_fsma_opts.IDLE
                case t_fsma_opts.WRITE:
                    smp.next = t_fsma_opts.IDLE
                case _:
                    smp.next = t_fsma_opts.IDLE

    return instances()


@block
def fsm4b(
    clk,
    a,
    b,
    c,
    z,
):

    @always(clk.posedge)
    def rtl():
        match concat(a, b, c):
            case intbv(0b111):
                z.next = 0x6
            case intbv(0b101) | intbv(0b110):
                z.next = 0x7
            case _:
                z.next = 0x0

    return instances()


@block
def muxBench0(setup=0):

    clk = Signal(bool(0))
    sel = Signal(intbv(0)[2:])
    in0 = Signal(intbv(0)[4:])
    in1 = Signal(intbv(0)[4:])
    in2 = Signal(intbv(0)[4:])
    in3 = Signal(intbv(0)[4:])
    out0 = Signal(intbv(0)[4:])

    @instance
    def clkgen():
        clk.next = 1
        for dummy in range(400):
            yield delay(10)
            clk.next = not clk

    @instance
    def stimulus():
        sel.next = 0x0
        in0.next = 0xa
        in1.next = 0xb
        in2.next = 0xc
        in3.next = 0xd
        yield clk.posedge
        yield clk.posedge
        sel.next = 0x1
        yield clk.posedge
        sel.next = 0x2
        yield clk.posedge
        sel.next = 0x3
        yield clk.posedge
        sel.next = 0x0
        yield clk.posedge

        raise StopSimulation

    @instance
    def check():
        yield clk.posedge
        yield clk.posedge
        assert out0 == 0xa
        yield clk.posedge
        assert out0 == 0xb
        yield clk.posedge
        assert out0 == 0xc
        yield clk.posedge
        assert out0 == 0xd
        yield clk.posedge
        assert out0 == 0xa
        yield clk.posedge

    if 0 == setup:
        i_mux = mux4a(sel, in0, in1, in2, in3, out0)
    elif 1 == setup:
        i_mux = mux4b(sel, in0, in1, in2, in3, out0)
    elif 2 == setup:
        i_mux = enumMux4a(sel, in0, in1, in2, in3, out0)
    else:
        i_mux = enumMux4b(sel, in0, in1, in2, in3, out0)

    return i_mux


def test_mux4a_convert():
    sel = Signal(intbv(0)[2:])
    in0 = Signal(intbv(0)[4:])
    in1 = Signal(intbv(0)[4:])
    in2 = Signal(intbv(0)[4:])
    in3 = Signal(intbv(0)[4:])
    out0 = Signal(intbv(0)[4:])

    i_dut = mux4a(sel, in0, in1, in2, in3, out0)
    assert i_dut.analyze_convert() == 0


def test_muxBench0():
    sim = muxBench0(0)
    sim.run_sim()


def test_muxBench0_convert():
    i_dut = muxBench0(0)
    assert i_dut.analyze_convert() == 0


# @pytest.mark.skipif(sys.version_info < (3, 10), reason="requires python3.10 or higher")
def test_mux4b_convert():
    sel = Signal(intbv(0)[2:])
    in0 = Signal(intbv(0)[4:])
    in1 = Signal(intbv(0)[4:])
    in2 = Signal(intbv(0)[4:])
    in3 = Signal(intbv(0)[4:])
    out0 = Signal(intbv(0)[4:])

    i_dut = mux4b(sel, in0, in1, in2, in3, out0)
    assert i_dut.analyze_convert() == 0


# @pytest.mark.skipif(sys.version_info < (3, 10), reason="requires python3.10 or higher")
def test_mux4c_convert():
    sel = Signal(intbv(0)[2:])
    in0 = Signal(intbv(0)[4:])
    in1 = Signal(intbv(0)[4:])
    in3 = Signal(intbv(0)[4:])
    out0 = Signal(intbv(0)[4:])

    i_dut = mux4c(sel, in0, in1, in3, out0)
    assert i_dut.analyze_convert() == 0


# @pytest.mark.skipif(sys.version_info < (3, 10), reason="requires python3.10 or higher")
def test_muxBench1():
    sim = muxBench0(1)
    sim.run_sim()


# @pytest.mark.skipif(sys.version_info < (3, 10), reason="requires python3.10 or higher")
def test_muxBench1_convert():
    i_dut = muxBench0(1)
    assert i_dut.analyze_convert() == 0


def test_muxBench2():
    sim = muxBench0(2)
    sim.run_sim()


def test_muxBench2_convert():
    i_dut = muxBench0(2)
    assert i_dut.analyze_convert() == 0


def test_enumMux4a_convert():
    sel = Signal(intbv(0)[2:])
    in0 = Signal(intbv(0)[4:])
    in1 = Signal(intbv(0)[4:])
    in2 = Signal(intbv(0)[4:])
    in3 = Signal(intbv(0)[4:])
    out0 = Signal(intbv(0)[4:])

    i_dut = enumMux4a(sel, in0, in1, in2, in3, out0)
    assert i_dut.analyze_convert() == 0


def test_muxBench3():
    sim = muxBench0(3)
    sim.run_sim()


def test_enumMux4b_convert():
    sel = Signal(intbv(0)[2:])
    in0 = Signal(intbv(0)[4:])
    in1 = Signal(intbv(0)[4:])
    in2 = Signal(intbv(0)[4:])
    in3 = Signal(intbv(0)[4:])
    out0 = Signal(intbv(0)[4:])

    i_dut = enumMux4b(sel, in0, in1, in2, in3, out0)
    assert i_dut.analyze_convert() == 0


def test_fsm4a_convert():
    clk = Signal(bool(0))
    rst = Signal(intbv(0)[1:])
    rd = Signal(intbv(0)[1:])
    wr = Signal(intbv(0)[1:])

    i_dut = fsm4a(clk, rst, rd, wr)
    assert i_dut.analyze_convert() == 0


def test_fsm4b_convert():
    clk = Signal(bool(0))
    a = Signal(intbv(0)[1:])
    b = Signal(intbv(0)[1:])
    c = Signal(intbv(0)[1:])
    z = Signal(intbv(0)[3:])

    i_dut = fsm4b(clk, a, b, c, z)
    assert i_dut.analyze_convert() == 0




import os
path = os.path
import unittest
from unittest import TestCase
import random
from random import randrange
random.seed(2)

from myhdl import (block, Signal, intbv, delay, always,
                   instance, StopSimulation)
from myhdl._Simulation import Simulation
from .util import setupCosimulation

ACTIVE_LOW, INACTIVE_HIGH = 0, 1


@block
def incRef(count, enable, clock, reset, n):
    """ Incrementer with enable.
    
    count -- output
    enable -- control input, increment when 1
    clock -- clock input
    reset -- asynchronous reset input
    n -- counter max value
    """

    @instance
    def comb():
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    count.next = (count + 1) % n

    return comb


@block
def inc(count, enable, clock, reset, n):

    """ Incrementer with enable.
    
    count -- output
    enable -- control input, increment when 1
    clock -- clock input
    reset -- asynchronous reset input
    n -- counter max value
    
    """

    @always(clock.posedge, reset.negedge)
    def incProcess():
        if reset == ACTIVE_LOW:
            count.next = 0
        else:
            if enable:
                count.next = (count + 1) % n

    return incProcess


@block
def inc2(count, enable, clock, reset, n):

    @always(clock.posedge, reset.negedge)
    def incProcess():
        if reset == ACTIVE_LOW:
            count.next = 0
        else:
            if enable:
                if count == n - 1:
                    count.next = 0
                else:
                    count.next = count + 1

    return incProcess


@block
def incTask(count, enable, clock, reset, n):

    def incTaskFunc(cnt, enable, reset, n):
        if enable:
            cnt[:] = (cnt + 1) % n

    @instance
    def incTaskGen():
        cnt = intbv(0)[8:]
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                cnt[:] = 0
                count.next = 0
            else:
                # print count
                incTaskFunc(cnt, enable, reset, n)
                count.next = cnt

    return incTaskGen


@block
def incTaskFreeVar(count, enable, clock, reset, n):

    def incTaskFunc():
        if enable:
            count.next = (count + 1) % n

    @always(clock.posedge, reset.negedge)
    def incTaskGen():
        if reset == ACTIVE_LOW:
            count.next = 0
        else:
            # print count
            incTaskFunc()

    return incTaskGen


@block
def inc_v(name, count, enable, clock, reset):
    return setupCosimulation(**locals())


class TestInc(TestCase):

    def clockGen(self, clock):
        while 1:
            yield delay(10)
            clock.next = not clock

    def stimulus(self, enable, clock, reset):
        reset.next = INACTIVE_HIGH
        yield clock.negedge
        reset.next = ACTIVE_LOW
        yield clock.negedge
        reset.next = INACTIVE_HIGH
        for dummy in range(1000):
            enable.next = 1
            yield clock.negedge
        for dummy in range(1000):
            enable.next = min(1, randrange(5))
            yield clock.negedge
        raise StopSimulation

    def check(self, count, count_v, enable, clock, reset, n):
        expect = 0
        yield reset.posedge
        self.assertEqual(count, expect)
        self.assertEqual(count, count_v)
        while 1:
            yield clock.posedge
            if enable:
                expect = (expect + 1) % n
            yield delay(1)
            # print "%d count %s expect %s count_v %s" % (now(), count, expect, count_v)
            self.assertEqual(count, expect)
            self.assertEqual(count, count_v)

    def bench(self, inc):

        m = 8
        n = 2 ** m

        count = Signal(intbv(0)[m:])
        count_v = Signal(intbv(0)[m:])
        enable = Signal(bool(0))
        clock, reset = [Signal(bool()) for __ in range(2)]

        inc_inst_ref = incRef(count, enable, clock, reset, n=n)
        inc_inst = inc(count, enable, clock, reset, n=n).convert(hdl='Verilog')
        # inc_inst = inc(count, enable, clock, reset, n=n)
        inc_inst_v = inc_v(inc.__name__, count_v, enable, clock, reset)
        clk_1 = self.clockGen(clock)
        st_1 = self.stimulus(enable, clock, reset)
        ch_1 = self.check(count, count_v, enable, clock, reset, n=n)

        sim = Simulation(inc_inst_ref, inc_inst_v, clk_1, st_1, ch_1)
        return sim

    def testIncRef(self):
        """ Check increment operation """
        sim = self.bench(incRef)
        sim.run(quiet=1)

    def testInc(self):
        """ Check increment operation """
        sim = self.bench(inc)
        sim.run(quiet=1)

    def testInc2(self):
        """ Check increment operation """
        sim = self.bench(inc2)
        sim.run(quiet=1)

    def testIncTask(self):
        sim = self.bench(incTask)
        sim.run(quiet=1)

    def testIncTaskFreeVar(self):
        sim = self.bench(incTaskFreeVar)
        sim.run(quiet=1)




import os
path = os.path
import unittest
from unittest import TestCase
import random
from random import randrange
random.seed(2)

from myhdl import (block, Signal, intbv, delay, downrange,
                   instance, StopSimulation)
from myhdl._Simulation import Simulation
from myhdl import ConversionError
from myhdl.conversion._misc import _error

ACTIVE_LOW, INACTIVE_HIGH = 0, 1


@block
def freeVarTypeError(count, enable, clock, reset, n):
    cnt = intbv(0)[8:]

    def incTaskFunc():
        if enable:
            cnt[:] = (cnt + 1) % n

    @instance
    def incTaskGen():
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                cnt[:] = 0
            else:
                incTaskFunc()

    return incTaskGen


@block
def multipleDrivenSignal(count, enable, clock, reset, n):

    @instance
    def incTaskGen():
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    count.next = (count + 1) % n

    return incTaskGen, incTaskGen


@block
def shadowingSignal(count, enable, clock, reset, n):
    count = Signal(intbv(0)[8:])

    @instance
    def incTaskGen():
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    count.next = (count + 1) % n

    return incTaskGen


@block
def internalSignal(count, enable, clock, reset, n):

    @instance
    def comb():
        a = Signal(bool())
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    count.next = (count + 1) % n

    return comb


@block
def undefinedBitWidthSignal(count, enable, clock, reset, n):
    count = Signal(intbv(0))

    @instance
    def incTaskGen():
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    count.next = (count + 1) % n

    return incTaskGen


@block
def yieldObject1(count, enable, clock, reset, n):

    @instance
    def comb():
        while 1:
            yield clock.posedge, delay(5)
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    count.next = (count + 1) % n

    return comb


def g1(clock):
        yield clock.posedge


def g2(reset):
        yield reset.negedge


@block
def yieldObject2(count, enable, clock, reset, n):

    @instance
    def comb():
        while 1:
            yield g1(clock), g2(reset)
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    count.next = (count + 1) % n

    return comb


def f1(n):
    if n == 0:
        return 0
    else:
        return f1(n - 1)


def f2(n):
    if n == 0:
        return 1
    else:
        return f3(n - 1)


def f3(n):
    if n == 0:
        return 1
    else:
        return f2(n - 1)


@block
def recursion1(count, enable, clock, reset, n):

    @instance
    def comb():
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    count.next = f1(n)

    return comb


@block
def recursion2(count, enable, clock, reset, n):

    @instance
    def comb():
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    count.next = f2(n)

    return comb


def h1(n):
    pass
    # return None


@block
def functionNoReturnVal(count, enable, clock, reset, n):

    @instance
    def comb():
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    count.next = h1(n)

    return comb


def h2(cnt):
    cnt[:] = cnt + 1
    return 1


@block
def taskReturnVal(count, enable, clock, reset, n):

    @instance
    def comb():
        cnt = intbv(0)[8:]
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    h2(cnt)
                    count.next = count + 1

    return comb


@block
def listComp1(count, enable, clock, reset, n):

    @instance
    def comb():
        mem = [intbv(0)[8:] for dummy in range(4) for dummy in range(5)]
        while 1:
            yield clock.posedge, reset.negedge
            count.next = count + 1

    return comb


@block
def listComp2(count, enable, clock, reset, n):

    @instance
    def comb():
        mem = [intbv(0)[8:] for dummy in downrange(4)]
        while 1:
            yield clock.posedge, reset.negedge
            count.next = count + 1

    return comb


@block
def listComp3(count, enable, clock, reset, n):

    @instance
    def comb():
        mem = [intbv(0)[8:] for dummy in range(1, 4)]
        while 1:
            yield clock.posedge, reset.negedge
            count.next = count + 1

    return comb


@block
def listComp4(count, enable, clock, reset, n):

    @instance
    def comb():
        mem = [intbv(0) for dummy in range(4)]
        while 1:
            yield clock.posedge, reset.negedge
            count.next = count + 1

    return comb


@block
def listComp5(count, enable, clock, reset, n):

    @instance
    def comb():
        mem = [i for i in range(4)]
        while 1:
            yield clock.posedge, reset.negedge
            count.next = count + 1

    return comb


@block
def undefinedBitWidthMem(count, enable, clock, reset, n):
    mem = [Signal(intbv(0)[8:]) for i in range(8)]
    mem[7] = Signal(intbv(0))

    @instance
    def f():
        while 1:
            yield clock.posedge, reset.negedge
            count.next = mem[0] + 1

    return f


@block
def inconsistentTypeMem(count, enable, clock, reset, n):
    mem = [Signal(intbv(0)[8:]) for __ in range(8)]
    mem[3] = Signal(bool())

    @instance
    def f():
        while 1:
            yield clock.posedge, reset.negedge
            count.next = mem[0] + 1

    return f


@block
def inconsistentBitWidthMem(count, enable, clock, reset, n):
    mem = [Signal(intbv(0)[8:]) for __ in range(8)]
    mem[4] = Signal(intbv(0)[7:])

    @instance
    def f():
        while 1:
            yield clock.posedge, reset.negedge
            count.next = mem[0] + 1

    return f

# # def listElementNotUnique(count, enable, clock, reset, n):
# #     mem = [Signal(intbv(0)[8:]) for i in range(8)]
# #     mem2 = mem[4:]
# #     def f():
# #         while 1:
# #             yield clock.posedge, reset.negedge
# #             count.next = mem[0] + mem2[1]
# #     return f()


class TestErr(TestCase):

    def clockGen(self, clock):
        while 1:
            yield delay(10)
            clock.next = not clock

    def stimulus(self, enable, clock, reset):
        reset.next = INACTIVE_HIGH
        yield clock.negedge
        reset.next = ACTIVE_LOW
        yield clock.negedge
        reset.next = INACTIVE_HIGH
        for dummy in range(1000):
            enable.next = 1
            yield clock.negedge
        for dummy in range(1000):
            enable.next = min(1, randrange(5))
            yield clock.negedge
        raise StopSimulation

    def check(self, count, count_v, enable, clock, reset, n):
        expect = 0
        yield reset.posedge
        self.assertEqual(count, expect)
        self.assertEqual(count, count_v)
        while 1:
            yield clock.posedge
            if enable:
                expect = (expect + 1) % n
            yield delay(1)
            # print "%d count %s expect %s count_v %s" % (now(), count, expect, count_v)
            self.assertEqual(count, expect)
            self.assertEqual(count, count_v)

    def bench(self, err):

        m = 8
        n = 2 ** m

        count = Signal(intbv(0)[m:])
        count_v = Signal(intbv(0)[m:])
        enable = Signal(bool(0))
        clock, reset = [Signal(bool()) for __ in range(2)]

        err_inst = err(count, enable, clock, reset, n=n).convert(hdl='Verilog')
        clk_1 = self.clockGen(clock)
        st_1 = self.stimulus(enable, clock, reset)
        ch_1 = self.check(count, count_v, enable, clock, reset, n=n)

        sim = Simulation(err_inst, clk_1, st_1, ch_1)
        return sim

    def testInternalSignal(self):
        try:
            self.bench(internalSignal)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.TypeInfer)
        else:
            self.fail()

    def testMultipleDrivenSignal(self):
        try:
            self.bench(multipleDrivenSignal)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.SigMultipleDriven)
        else:
            self.fail()

    def testShadowingSignal(self):
        try:
            self.bench(shadowingSignal)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.ShadowingSignal)
        else:
            self.fail()

    def testUndefinedBitWidthSignal(self):
        try:
            self.bench(undefinedBitWidthSignal)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.UndefinedBitWidth)
        else:
            self.fail()

    def testFreeVarTypeError(self):
        try:
            self.bench(freeVarTypeError)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.FreeVarTypeError)
        else:
            self.fail()

# #     def testNegIntbv(self):
# #         try:
# #             self.bench(negIntbv)
# #         except ConversionError, e:
# #             self.assertEqual(e.kind, _error.IntbvSign)
# #         else:
# #             self.fail()

    def testYield1(self):
        try:
            self.bench(yieldObject1)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.UnsupportedYield)
        else:
            self.fail()

    def testYield2(self):
        try:
            self.bench(yieldObject2)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.NotSupported)
        else:
            self.fail()

    def testRecursion1(self):
        try:
            self.bench(recursion1)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.NotSupported)
        else:
            self.fail()

    def testRecursion2(self):
        try:
            self.bench(recursion2)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.NotSupported)
        else:
            self.fail()

    def testFunctionNoReturnVal(self):
        try:
            self.bench(functionNoReturnVal)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.NotSupported)
        else:
            self.fail()

    def testTaskReturnVal(self):
        try:
            self.bench(taskReturnVal)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.NotSupported)
        else:
            self.fail()

    def testListComp1(self):
        try:
            self.bench(listComp1)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.NotSupported)
        else:
            self.fail()

    def testListComp2(self):
        try:
            self.bench(listComp2)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.UnsupportedListComp)
        else:
            self.fail()

    def testListComp3(self):
        try:
            self.bench(listComp3)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.UnsupportedListComp)
        else:
            self.fail()

    def testListComp4(self):
        try:
            self.bench(listComp4)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.UnsupportedListComp)
        else:
            self.fail()

    def testListComp5(self):
        try:
            self.bench(listComp5)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.UnsupportedListComp)
        else:
            self.fail()

    def testUndefinedBitWidthMem(self):
        try:
            self.bench(undefinedBitWidthMem)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.UndefinedBitWidth)
        else:
            self.fail()

    def testInconsistentTypeMem(self):
        try:
            self.bench(inconsistentTypeMem)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.InconsistentType)
        else:
            self.fail()

    def testInconsistentBitWidthMem(self):
        try:
            self.bench(inconsistentBitWidthMem)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.InconsistentBitWidth)
        else:
            self.fail()

# #     def testListElementNotUnique(self):
# #         try:
# #             self.bench(listElementNotUnique)
# #         except ConversionError, e:
# #             self.assertEqual(e.kind, _error.ListElementNotUnique)
# #         else:
# #             self.fail()


if __name__ == '__main__':
    unittest.main()




import os
path = os.path
import random
random.seed(2)

from myhdl import (block, intbv, instance, delay)

ACTIVE_LOW, INACTIVE_HIGH = bool(0), bool(1)


@block
def bug_1835797():
    """ Docstring in the middle.

    """

    @instance
    def comb():
        v = intbv(0, min=-15, max=45)
        """Wrongly placed docstring"""
        yield delay(10)
        print(v.min)
        print(v.max)

    return comb


def test_bug_1835797():
    assert bug_1835797().verify_convert() == 0




#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2011 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Run the modbv unit tests. """
import sys
import random
import pytest

from random import randrange

from myhdl import intbv, modbv

random.seed(3)
maxint = sys.maxsize


class TestModbvWrap:

    def testWrap(self):
        x = modbv(0, min=-8, max=8)
        x[:] = x + 1
        assert 1 == x
        x[:] = x + 2
        assert 3 == x
        x[:] = x + 5
        assert -8 == x
        x[:] = x + 1
        assert -7 == x
        x[:] = x - 5
        assert 4 == x
        x[:] = x - 4
        assert 0 == x
        x[:] += 15
        x[:] = x - 1
        assert -2 == x

    def testInit(self):
        with pytest.raises(ValueError):
            intbv(15, min=-8, max=8)

        x = modbv(15, min=-8, max=8)
        assert -1 == x

        # Arbitrary boundraries support (no exception)
        modbv(5, min=-3, max=8)

    def testNoWrap(self):
        # Validate the base class fails for the wraps
        x = intbv(0, min=-8, max=8)
        with pytest.raises(ValueError):
            x[:] += 15

        x = intbv(0, min=-8, max=8)
        with pytest.raises(ValueError):
            x[:] += 15


class TestModBvSlice:

    def BuildTestSeqs(self):
        imin, jmin = 0, 0
        imax, jmax = 5000, 5000
        seqi, seqj = [], []
        # small bits
        for __ in range(10):
            seqi.append(randrange(imin, imax))
            seqi.append(randrange(jmin, jmax))
        # large bits
        for __ in range(10):
            seqi.append(randrange(imin, maxint))
            seqj.append(randrange(jmin, maxint))
        self.seqi = seqi
        self.seqj = seqj

    def testGetItem(self):
        self.BuildTestSeqs()
        # Testing for the __getItem__ function
        for i in self.seqi:
            ti = modbv(i)
            cti = modbv(~i)
            for n in range(len(bin(i))):
                chk = ((i >> n) & 1)
                res = ti[n]
                resv = cti[n]
                assert type(res) == bool
                assert type(resv) == bool
                assert res == chk
                assert resv == chk ^ 1

    def testGetSlice(self):
        self.BuildTestSeqs()
        # Testing for slice ranged, both closed
        for i in self.seqi:
            ti = modbv(i)
            cti = modbv(~i)
            leni = len(bin(i))
            for __ in (range(leni)):
                s = randrange(-leni, leni)
                f = randrange(-leni, leni)
                try:
                    val = ti[f:s]
                except ValueError:
                    assert (f <= s) | (s <= 0)
                else:
                    chkt = (i >> s) & (2 ** (f - s) - 1)
                    assert val == chkt
                    assert (val.max, val.min) == (2 ** (f - s), 0)

    def testGetSliceOpen(self):
        self.BuildTestSeqs()
        # Testing for slice ranged, one of them open
        for i in self.seqi:
            ti = modbv(i)
            cti = modbv(~i)
            leni = len(bin(i))
            assert ti + cti == -1
            for __ in (range(leni)):
                s = randrange(leni)
                f = randrange(1, leni)
                chkf = (i) & (2 ** (f) - 1)
                chks = (i >> s)
                vals, cvals = ti[:s], cti[:s]
                valf, cvalf = ti[f:], cti[f:]
                assert vals + cvals == -1

                # assert valf+cvalf == -1   ---test fails- why???
                assert vals == modbv(chks)
                assert valf == modbv(chkf)

    def testSetSlice(self):
        self.BuildTestSeqs()
        self.seqi.extend(self.seqj)
        # Test the bit slice setting wrap around behaviour
        for i in range(1, len(self.seqi)):
            lenp = len(bin(self.seqi[i - 1]))
            ti = modbv(self.seqi[i - 1])[lenp:]
            maxp = 2 ** lenp
            vali = self.seqi[i]
            s = randrange(lenp - 1)
            f = randrange(s, lenp)
            valp = self.seqi[i - 1]
            maxval = (1 << (f - s)) - 1
            try:
                ti[f:s] = vali
            except ValueError:
                assert maxval < vali
            else:
                bitmask = (((2 ** f - 1) >> s) << s)
                valp = valp - (valp & bitmask)
                valp = (valp + ((vali << s) % maxp)) % maxp
                assert valp == ti




from myhdl import (block, Signal, always, always_comb)
from myhdl import ToVHDLWarning

import pytest
import tempfile
import shutil
import sys
import string
import importlib
import os
from keyword import kwlist as python_kwlist

import warnings

_vhdl_keywords = ["abs", "access", "after", "alias", "all",
                  "and", "architecture", "array", "assert",
                  "attribute", "begin", "block", "body", "buffer",
                  "bus", "case", "component", "configuration",
                  "constant", "disconnect", "downto", "else",
                  "elseif", "end", "entity", "exit", "file", "for",
                  "function", "generate", "generic", "group",
                  "guarded", "if", "impure", "in", "inertial",
                  "inout", "is", "label", "library", "linkage",
                  "literal", "loop", "map", "mod", "nand", "new",
                  "next", "nor", "not", "null", "of", "on", "open",
                  "or", "others", "out", "package", "port",
                  "postponed", "procedure", "process", "pure",
                  "range", "record", "register", "reject", "rem",
                  "report", "return", "rol", "ror", "select",
                  "severity", "signal", "shared", "sla", "sll", "sra",
                  "srl", "subtype", "then", "to", "transport", "type",
                  "unaffected", "units", "until", "use", "variable",
                  "wait", "when", "while", "with", "xnor", "xor"];

keyword_code = """
from myhdl import *

@block
def invalid_import_keyword(input_sig, output_sig):
    ${keyword} = Signal(False)

    @always_comb
    def do_something():
        ${keyword}.next = input_sig and input_sig

    @always_comb
    def something_else():
        output_sig.next = ${keyword}

    return do_something, something_else
"""


@block
def invalid_signal_underscore(input_sig, output_sig):
    _foo = Signal(bool(0))

    @always_comb
    def do_something():
        _foo.next = input_sig and input_sig

    @always_comb
    def something_else():
        output_sig.next = _foo

    return do_something, something_else


@block
def invalid_function_underscore(clock, input_sig, output_sig):

    ttt = Signal(bool(0))

    block1 = invalid_signal_underscore(input_sig, ttt)

    @always(clock.posedge)
    def do_something():
        output_sig.next = ttt

    return block1, do_something


@block
def valid(input_sig, output_sig):

    @always_comb
    def do_something():
        output_sig.next = input_sig

    return do_something


def test_multiple_conversion():
    sig_1 = Signal(True)
    sig_2 = Signal(True)

    a_block = valid(sig_1, sig_2)

    # conversions with keyword should fail
    with warnings.catch_warnings() as w:
        warnings.simplefilter('error')

        a_block.convert(hdl='VHDL')
        a_block.convert(hdl='VHDL')


def test_invalid_keyword_name():
    sig_1 = Signal(True)
    sig_2 = Signal(True)

    temp_directory = tempfile.mkdtemp()
    sys.path.append(temp_directory)

    keyword_template = string.Template(keyword_code)

    try:
        for keyword in _vhdl_keywords:
            if keyword in python_kwlist:
                continue

            fd, full_filename = tempfile.mkstemp(
                suffix='.py', dir=temp_directory)

            os.write(fd, keyword_template.substitute(keyword=keyword).encode('utf-8'))
            os.close(fd)

            module_name = os.path.basename(full_filename)[:-3]  # chop off .py
            keyword_import = importlib.import_module(module_name)

            a_block = keyword_import.invalid_import_keyword(sig_1, sig_2)

            with pytest.warns(ToVHDLWarning):
                a_block.convert(hdl='VHDL')

    finally:
        sys.path.pop()
        shutil.rmtree(temp_directory)


def test_invalid_signal_underscore_name():
    sig_1 = Signal(True)
    sig_2 = Signal(True)

    a_block = invalid_signal_underscore(sig_1, sig_2)

    # Multiple conversions of a valid block should pass without warning
    with pytest.warns(ToVHDLWarning):
        a_block.convert(hdl='VHDL')


def test_invalid_function_underscore_name():
    sig_1 = Signal(True)
    sig_2 = Signal(True)
    clock = Signal(True)

    a_block = invalid_function_underscore(clock, sig_1, sig_2)

    # Multiple conversions of a valid block should pass without warning
    with pytest.warns(ToVHDLWarning):
        a_block.convert(hdl='VHDL')


if __name__ == '__main__':
    sig_1 = Signal(True)
    sig_2 = Signal(True)

    a_block = invalid_signal_underscore(sig_1, sig_2)
    a_block.convert(hdl='VHDL')

    clock = Signal(True)

    a_block = invalid_function_underscore(clock, sig_1, sig_2)

    # Multiple conversions of a valid block should pass without warning
    a_block.convert(hdl='VHDL')



import myhdl
from myhdl import *

ACTIVE = 0
DirType = enum('RIGHT', 'LEFT')


def jc2_alt(goLeft, goRight, stop, clk, q):

    """ A bi-directional 4-bit Johnson counter with stop control.

    I/O pins:
    --------
    clk      : input free-running slow clock 
    goLeft   : input signal to shift left (active-low switch)
    goRight  : input signal to shift right (active-low switch)
    stop     : input signal to stop counting (active-low switch)
    q        : 4-bit counter output (active-low LEDs; q[0] is right-most)

    Operation:
    ---------
    The counter is triggered on the rising edge of the clock (clk). 
    A low pulse on the goLeft input will cause the counter to start 
    shifting left from its current state. A low pulse on the goRight
    input will cause the counter to start shifting right from its 
    current state. A low pulse on the stop input will cause the 
    counter to hold its current state until goLeft or goRight is pulsed.

    After power-up, the counter is stopped with all outputs low (LEDs lit).

    """

    @instance
    def comb():
        dir = DirType.LEFT
        run = False
        while True:
            yield clk.posedge
            # direction
            if goRight == ACTIVE:
                dir = DirType.RIGHT
                run = True
            elif goLeft == ACTIVE:
                dir = DirType.LEFT
                run = True
            # stop
            if stop == ACTIVE:
                run = False
            # counter action
            if run:
                if dir == DirType.LEFT:
                    q.next[4:1] = q[3:]
                    q.next[0] = not q[3]
                else:
                    q.next[3:] = q[4:1]
                    q.next[3] = not q[0]

    return comb



import os
path = os.path
import unittest
from unittest import TestCase
from random import randrange

from myhdl import (block, Signal, intbv, delay, instance, always_comb, StopSimulation)
from myhdl._Simulation import Simulation

from .util import setupCosimulation

D = 256

ROM = tuple([randrange(D) for __ in range(D)])
# ROM = [randrange(256) for i in range(256)]


@block
def rom1(dout, addr, clk):

    @instance
    def rdLogic():
        while 1:
            yield clk.posedge
            dout.next = ROM[int(addr)]

    return rdLogic


@block
def rom2(dout, addr, clk):

    theROM = ROM

    @instance
    def rdLogic():
        while 1:
            yield clk.posedge
            dout.next = theROM[int(addr)]

    return rdLogic


@block
def rom3(dout, addr, clk):

    @instance
    def rdLogic():
        tmp = intbv(0)[8:]
        while 1:
            yield addr
            tmp[:] = ROM[int(addr)]
            dout.next = tmp

    return rdLogic


@block
def rom4(dout, addr, clk):

    @always_comb
    def read():
        dout.next = ROM[int(addr)]

    return read


@block
def rom_v(name, dout, addr, clk):
    return setupCosimulation(**locals())


class TestRom(TestCase):

    def bench(self, rom):

        dout = Signal(intbv(0)[8:])
        dout_v = Signal(intbv(0)[8:])
        addr = Signal(intbv(1)[8:])
        clk = Signal(bool(0))

        rom_inst = rom(dout, addr, clk).convert(hdl='Verilog')
        rom_v_inst = rom_v(rom.__name__, dout_v, addr, clk)

        def stimulus():
            for i in range(D):
                addr.next = i
                yield clk.negedge
                yield clk.posedge
                yield delay(1)
                self.assertEqual(dout, ROM[i])
                self.assertEqual(dout, dout_v)
            raise StopSimulation()

        def clkgen():
            while 1:
                yield delay(10)
                clk.next = not clk

        return clkgen(), stimulus(), rom_inst, rom_v_inst

    def test1(self):
        sim = self.bench(rom1)
        Simulation(sim).run()

    def test2(self):
        sim = self.bench(rom2)
        Simulation(sim).run()

    def test3(self):
        sim = self.bench(rom3)
        Simulation(sim).run()

    def test4(self):
        sim = self.bench(rom4)
        Simulation(sim).run()


if __name__ == '__main__':
    unittest.main()




from myhdl import (block, Signal, always_comb)

INT_CONDITION_0 = 0
INT_CONDITION_1 = 1
BOOL_CONDITION_0 = False
BOOL_CONDITION_1 = True


@block
def bug_boolconst(sigin, sigout):

    @always_comb
    def comb():
        sigout.next = 0
        if INT_CONDITION_0:
            sigout.next = sigin
        if BOOL_CONDITION_0:
            sigout.next = sigin
        if not INT_CONDITION_0:
            sigout.next = sigin
        if not BOOL_CONDITION_0:
            sigout.next = sigin
        if INT_CONDITION_1:
            sigout.next = sigin
        if BOOL_CONDITION_1:
            sigout.next = sigin
        if not INT_CONDITION_1:
            sigout.next = sigin
        if not BOOL_CONDITION_1:
            sigout.next = sigin

    return comb


def test_bug_boolconst():
    sigin = Signal(bool())
    sigout = Signal(bool())

    assert bug_boolconst(sigin, sigout).analyze_convert() == 0




from myhdl import (block, Signal, intbv, modbv, delay, always_comb, instance, instances, StopSimulation)


@block
def binOpsCheck0(
    a,
    b,
    c,
    x,
    y,
    z,
):

    @always_comb
    def outputs():
        x.next = a
        y.next = a | b
        z.next = a | b | c

    return instances()


@block
def binOpsCheck1(
    a,
    x,
    y,
    z,
):

    @always_comb
    def outputs():
        x.next = a[0]
        y.next = a[0] | a[1]
        z.next = a[0] | a[1] | a[2]

    return instances()


@block
def binOpsCheck2(
    a,
    z,
):

    @always_comb
    def outputs():
        z[0].next = a[0]
        z[1].next = a[0] | a[1]
        z[2].next = a[0] | a[1] | a[2]

    return instances()


@block
def binOpsCheckBench0():

    clk = Signal(bool(0))
    a = Signal(modbv(0)[1:])
    b = Signal(modbv(0)[1:])
    c = Signal(modbv(0)[1:])
    x = Signal(modbv(0)[1:])
    y = Signal(modbv(0)[1:])
    z = Signal(modbv(0)[1:])

    @instance
    def clkgen():
        clk.next = 1
        for dummy in range(400):
            yield delay(10)
            clk.next = not clk

    @instance
    def stimulus():
        a.next = 0x0
        b.next = 0x0
        c.next = 0x0
        yield clk.posedge
        yield clk.posedge
        a.next = 0x0
        b.next = 0x0
        c.next = 0x0
        yield clk.posedge
        a.next = 0x1
        b.next = 0x0
        c.next = 0x0
        yield clk.posedge
        a.next = 0x0
        b.next = 0x1
        c.next = 0x0
        yield clk.posedge
        a.next = 0x1
        b.next = 0x1
        c.next = 0x0
        yield clk.posedge
        a.next = 0x0
        b.next = 0x0
        c.next = 0x1
        yield clk.posedge
        a.next = 0x1
        b.next = 0x0
        c.next = 0x1
        yield clk.posedge
        a.next = 0x0
        b.next = 0x1
        c.next = 0x0
        yield clk.posedge
        a.next = 0x1
        b.next = 0x1
        c.next = 0x1
        yield clk.posedge
        a.next = 0x0
        b.next = 0x0
        c.next = 0x0
        yield clk.posedge

        raise StopSimulation

    @instance
    def check():
        yield clk.posedge
        yield clk.posedge
        yield clk.posedge
        assert x == 0x0
        assert y == 0x0
        assert z == 0x0
        yield clk.posedge
        assert x == 0x1
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x0
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x1
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x0
        assert y == 0x0
        assert z == 0x1
        yield clk.posedge
        assert x == 0x1
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x0
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x1
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x0
        assert y == 0x0
        assert z == 0x0
        yield clk.posedge

    i_binOpsCheck = binOpsCheck0(a, b, c, x, y, z)

    return instances()


@block
def binOpsCheckBench1():

    clk = Signal(bool(0))
    a = Signal(modbv(0)[1:])
    b = Signal(modbv(0)[1:])
    c = Signal(modbv(0)[1:])
    x = Signal(False)
    y = Signal(False)
    z = Signal(False)

    @instance
    def clkgen():
        clk.next = 1
        for dummy in range(400):
            yield delay(10)
            clk.next = not clk

    @instance
    def stimulus():
        a.next = 0x0
        b.next = 0x0
        c.next = 0x0
        yield clk.posedge
        yield clk.posedge
        a.next = 0x0
        b.next = 0x0
        c.next = 0x0
        yield clk.posedge
        a.next = 0x1
        b.next = 0x0
        c.next = 0x0
        yield clk.posedge
        a.next = 0x0
        b.next = 0x1
        c.next = 0x0
        yield clk.posedge
        a.next = 0x1
        b.next = 0x1
        c.next = 0x0
        yield clk.posedge
        a.next = 0x0
        b.next = 0x0
        c.next = 0x1
        yield clk.posedge
        a.next = 0x1
        b.next = 0x0
        c.next = 0x1
        yield clk.posedge
        a.next = 0x0
        b.next = 0x1
        c.next = 0x0
        yield clk.posedge
        a.next = 0x1
        b.next = 0x1
        c.next = 0x1
        yield clk.posedge
        a.next = 0x0
        b.next = 0x0
        c.next = 0x0
        yield clk.posedge

        raise StopSimulation

    @instance
    def check():
        yield clk.posedge
        yield clk.posedge
        yield clk.posedge
        assert x == 0x0
        assert y == 0x0
        assert z == 0x0
        yield clk.posedge
        assert x == 0x1
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x0
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x1
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x0
        assert y == 0x0
        assert z == 0x1
        yield clk.posedge
        assert x == 0x1
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x0
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x1
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x0
        assert y == 0x0
        assert z == 0x0
        yield clk.posedge

    i_binOpsCheck = binOpsCheck0(a, b, c, x, y, z)

    return instances()


@block
def binOpsCheckBench2():

    clk = Signal(bool(0))
    a = Signal(modbv(0)[2:])
    b = Signal(modbv(0)[2:])
    c = Signal(modbv(0)[2:])
    x = Signal(modbv(0)[2:])
    y = Signal(modbv(0)[2:])
    z = Signal(modbv(0)[2:])

    @instance
    def clkgen():
        clk.next = 1
        for dummy in range(400):
            yield delay(10)
            clk.next = not clk

    @instance
    def stimulus():
        a.next = 0x0
        b.next = 0x0
        c.next = 0x0
        yield clk.posedge
        yield clk.posedge
        a.next = 0x0
        b.next = 0x0
        c.next = 0x0
        yield clk.posedge
        a.next = 0x1
        b.next = 0x0
        c.next = 0x0
        yield clk.posedge
        a.next = 0x0
        b.next = 0x1
        c.next = 0x0
        yield clk.posedge
        a.next = 0x1
        b.next = 0x1
        c.next = 0x0
        yield clk.posedge
        a.next = 0x0
        b.next = 0x0
        c.next = 0x1
        yield clk.posedge
        a.next = 0x1
        b.next = 0x0
        c.next = 0x1
        yield clk.posedge
        a.next = 0x0
        b.next = 0x1
        c.next = 0x0
        yield clk.posedge
        a.next = 0x1
        b.next = 0x1
        c.next = 0x1
        yield clk.posedge
        a.next = 0x0
        b.next = 0x0
        c.next = 0x0
        yield clk.posedge

        raise StopSimulation

    @instance
    def check():
        yield clk.posedge
        yield clk.posedge
        yield clk.posedge
        assert x == 0x0
        assert y == 0x0
        assert z == 0x0
        yield clk.posedge
        assert x == 0x1
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x0
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x1
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x0
        assert y == 0x0
        assert z == 0x1
        yield clk.posedge
        assert x == 0x1
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x0
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x1
        assert y == 0x1
        assert z == 0x1
        yield clk.posedge
        assert x == 0x0
        assert y == 0x0
        assert z == 0x0
        yield clk.posedge

    i_binOpsCheck = binOpsCheck0(a, b, c, x, y, z)

    return instances()


def test_binOps0():
    sim = binOpsCheckBench0()
    sim.run_sim()


def test_binOps0_convert():
    i_dut = binOpsCheckBench0()
    assert i_dut.analyze_convert() == 0
    # assert i_dut.verify_simulator() == 0
    # assert i_dut.verify_convert() == 0


def test_binOps1():
    sim = binOpsCheckBench1()
    sim.run_sim()


def test_binOps1_convert():
    i_dut = binOpsCheckBench1()
    assert i_dut.analyze_convert() == 0
    # assert i_dut.verify_simulator() == 0


def test_binOps1b_convert():
    a = Signal(modbv(0)[1:])
    b = Signal(modbv(0)[1:])
    c = Signal(modbv(0)[1:])
    x = Signal(False)
    y = Signal(False)
    z = Signal(False)

    i_dut = binOpsCheck0(a, b, c, x, y, z)
    assert i_dut.analyze_convert() == 0


def test_binOps1c_convert():
    a = Signal(intbv(0)[1:])
    b = Signal(intbv(0)[1:])
    c = Signal(intbv(0)[1:])
    x = Signal(False)
    y = Signal(False)
    z = Signal(False)

    i_dut = binOpsCheck0(a, b, c, x, y, z)
    assert i_dut.analyze_convert() == 0


def test_binOps1d_convert():
    a = Signal(intbv(0)[3:])
    x = Signal(False)
    y = Signal(False)
    z = Signal(False)

    i_dut = binOpsCheck1(a, x, y, z)
    assert i_dut.analyze_convert() == 0


def test_binOps1e_convert():
    a = Signal(intbv(0)[3:])
    z = Signal(intbv(0)[3:])

    i_dut = binOpsCheck2(a, z)
    assert i_dut.analyze_convert() == 0


def test_binOps2():
    sim = binOpsCheckBench2()
    sim.run_sim()


def test_binOps2_convert():
    i_dut = binOpsCheckBench2()
    assert i_dut.analyze_convert() == 0
    # assert i_dut.verify_convert() == 0
    # assert i_dut.verify_simulator() == 0


if __name__ == '__main__':
    a = Signal(intbv(0)[3:])
    z = Signal(intbv(0)[3:])

    i_dut = binOpsCheck2(a, z)
    i_dut.convert(hdl='Verilog')



import myhdl
from myhdl import *

from glibc_random import glibc_random

from long_divider import long_divider

def test_longdiv(nrvectors=2**18):
    quotient = Signal(intbv(0)[22:])
    ready = Signal(bool())
    dividend = Signal(intbv(0)[38:])
    divisor = Signal(intbv(0)[16:])
    start = Signal(bool())
    clock = Signal(bool())
    reset = Signal(bool())
    stopped = Signal(bool())

    MAXVAL = 2**22 - 1
    
    dut = long_divider(
        quotient,
        ready,
        dividend,
        divisor,
        start,
        clock,
        reset
        )

    @instance
    def clockgen():
        clock.next = 0
        yield delay(10)
        while not stopped:
            clock.next = not clock
            yield delay(10)
        
    @instance
    def stimulus():
        stopped.next = 0
        yield delay(10)
        random_word = intbv(0)[32:]
        p = intbv(0)[16:]
        q = intbv(0)[22:]
        d = intbv(0)[38:]
        yield clock.negedge
        reset.next = 0
        yield clock.negedge
        reset.next = 1
        yield clock.negedge
        reset.next = 0
        start.next = 0
        yield clock.negedge
        random_word[:] = 94
        for i in range(nrvectors):
            yield clock.negedge
            random_word[:] = glibc_random(random_word)
            p[:] = random_word[16:]
            random_word[:] = glibc_random(random_word)
            q[:] = random_word[22:]
            if p == 0:
                q[:] = MAXVAL
            d[:] = p * q
            dividend.next = d
            divisor.next = p
            start.next = 1
            yield clock.negedge
            start.next = 0
            yield ready.posedge
            """compensate for Verilog's non-determinism"""
            yield delay(1) 
            #print d, p, q, quotient
            assert quotient == q
        stopped.next = 1 
        yield delay(10)
        #raise StopSimulation()
            
    return dut, clockgen, stimulus
    
if __name__ == '__main__':    
    sim = Simulation(test_longdiv())
    sim.run()




#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Run the unit tests for inferWaiter """
import random
from random import randrange
random.seed(1) # random, but deterministic

import myhdl
from myhdl import *
from myhdl._Waiter import _SignalWaiter,_SignalTupleWaiter, _DelayWaiter, \
                          _EdgeWaiter, _EdgeTupleWaiter, _Waiter
from test_inferWaiter import *



QUIET=1


N=10

def bench(genFunc, waiterType):

    a, b, c, d, r, s = [Signal(intbv()) for i in range(6)]

    s = [Signal(intbv()) for i in range(N)]
    gen_inst_s = []

    for i in range(N):
        gen_inst_s.append(waiterType(genFunc(a, b, c, d, s[i])))

    def stimulus():
        for i in range(5000):
            yield delay(randrange(1, 10))
            if randrange(2):
                a.next = randrange(32)
            if randrange(2):
                b.next = randrange(32)
            c.next = randrange(2)
            d.next = randrange(2)
        raise StopSimulation()

    return gen_inst_s, _Waiter(stimulus())


gen = SignalGen2

waiter = _Waiter

sim = Simulation(bench(gen, waiter))
sim.run()





'''
Created on 21 dec. 2024

@author: josy
'''

from myhdl import HdlClass, block, Signal, always_comb


class Minimal(HdlClass):
    ''' a minimal class design to exercise/debug the conversion flow '''

    def __init__(self, Sigin, SigOut=None):
        self.Sigin = Sigin
        self.SigOut = SigOut if SigOut is not None else Sigin.duplicate()

    @block(skipname=True)
    def hdl(self):

        @always_comb
        def comb():
            self.SigOut.next = ~self.Sigin

        return self.hdlinstances()


if __name__ == '__main__':
    if 0:
        Sigin = Signal(bool(0))
        SigOut = Signal(bool(0))

        # dfc = Minimal(Sigin, SigOut)
        dfc = Minimal(Sigin)
        dfc.convert(hdl='Verilog', name='Minimal')
        dfc.convert(hdl='VHDL', name='Minimal')

    else:

        def convert():
            Sigin = Signal(bool(0))
            SigOut = Signal(bool(0))

            dfc = Minimal(Sigin)
            dfc.convert(hdl='VHDL', name='Minimal')
            dfc.convert(hdl='Verilog', name='Minimal')

        convert()



import unittest
from unittest import TestCase
import random
from random import randrange
random.seed(2)

#import psyco
#psyco.profile()

from myhdl import Simulation, StopSimulation, Signal, \
                  delay, intbv, negedge, posedge, now

from inc import inc

ACTIVE_LOW, INACTIVE_HIGH = 0, 1

class TestInc(TestCase):

    def clockGen(self, clock):
        while 1:
            yield delay(10)
            clock.next = not clock
    
    def stimulus(self, enable, clock, reset):
        reset.next = ACTIVE_LOW
        yield negedge(clock)
        reset.next = INACTIVE_HIGH
        for i in range(1000):
            enable.next = min(1, randrange(5))
            yield negedge(clock)
        raise StopSimulation

    def check(self, count, enable, clock, reset, n):
        expect = 0
        yield posedge(reset)
        self.assertEqual(count, expect)
        while 1:
            yield posedge(clock)
            if enable:
                expect = (expect + 1) % n
            yield delay(1)
            # print "%d count %s expect %s" % (now(), count, expect)
            self.assertEqual(count, expect)
                
    def bench(self):

        n = 253

        count, enable, clock, reset = [Signal(intbv(0)) for i in range(4)]

        INC_1 = inc(count, enable, clock, reset, n=n)
        CLK_1 = self.clockGen(clock)
        ST_1 = self.stimulus(enable, clock, reset)
        CH_1 = self.check(count, enable, clock, reset, n=n)

        sim = Simulation(INC_1, CLK_1, ST_1, CH_1)
        return sim

    def test1(self):
        """ Check increment operation """
        sim = self.bench()
        sim.run(quiet=1)
        
    def test2(self):
        """ Check increment operation with suspended simulation runs """
        sim = self.bench()
        while sim.run(duration=randrange(1, 6), quiet=1):
            pass

          
if __name__ == '__main__':
    unittest.main()


            
            

    

    
        


                

        




import os
path = os.path
import unittest
from unittest import TestCase
import random
from random import randrange
random.seed(2)

from myhdl import (block, Signal, intbv, delay, instance, always, StopSimulation)
from myhdl._Simulation import Simulation

from .util import setupCosimulation


@block
def binaryOps(
# #                 Bitand,
# #               Bitor,
# #               Bitxor,
# #               FloorDiv,
                 LeftShift,
# #               Mod,
                 Mul,
# #               Pow,
                 RightShift,
                 Sub,
                 Sum, Sum1, Sum2, Sum3,
                 EQ,
                 NE,
                 LT,
                 GT,
                 LE,
                 GE,
                 And,
                 Or,
                 left, right, bit):

    @instance
    def comb():
        while 1:
            yield left, right
    # #        Bitand.next = left & right
    # #         Bitor.next = left | right
    # #         Bitxor.next = left ^ right
    # #         if right != 0:
    # #             FloorDiv.next = left // right
            if left < 256 and right < 40  and right >= 0:
                LeftShift.next = left << right
    # #         if right != 0:
    # #             Mod.next = left % right
            Mul.next = left * right
    # #         # Icarus doesn't support ** yet
    # #         #if left < 256 and right < 40:
    # #         #    Pow.next = left ** right
    # #         Pow.next = 0
            if right >= -0:
                RightShift.next = left >> right
                # # RightShift.next = left
            Sub.next = left - right
            Sum.next = left + right
            Sum1.next = left + right[2:]
            Sum2.next = left + right[1]
            Sum3.next = left + bit
            EQ.next = left == right
            NE.next = left != right
            LT.next = left < right
            GT.next = left > right
            LE.next = left <= right
            GE.next = left >= right
            And.next = bool(left) and bool(right)
            Or.next = bool(left) or bool(right)

    return comb


@block
def binaryOps_v(name,
# #                Bitand,
# #                 Bitor,
# #                 Bitxor,
# #                 FloorDiv,
                LeftShift,
# #                 Mod,
                Mul,
# #                 Pow,
                RightShift,
                Sub,
                Sum, Sum1, Sum2, Sum3,
                EQ,
                NE,
                LT,
                GT,
                LE,
                GE,
                And,
                Or,
                left, right, bit):
    return setupCosimulation(**locals())


class TestBinaryOps(TestCase):

    def binaryBench(self, Ll, Ml, Lr, Mr):

        bit = Signal(bool(0))
        left = Signal(intbv(Ll, min=Ll, max=Ml))
        right = Signal(intbv(Lr, min=Lr, max=Mr))
        M = 2 ** 14
# #        Bitand = Signal(intbv(0, min=-2**17, max=2**17))
# #        Bitand_v = Signal(intbv(0, min=-2**17, max=2**17))
# #         Bitor = Signal(intbv(0)[max(m, n):])
# #         Bitor_v = Signal(intbv(0)[max(m, n):])
# #         Bitxor = Signal(intbv(0)[max(m, n):])
# #         Bitxor_v = Signal(intbv(0)[max(m, n):])
# #         FloorDiv = Signal(intbv(0)[m:])
# #         FloorDiv_v = Signal(intbv(0)[m:])
        LeftShift = Signal(intbv(0, min=-2 ** 64, max=2 ** 64))
        LeftShift_v = Signal(intbv(0, min=-2 ** 64, max=2 ** 64))
# #         Mod = Signal(intbv(0)[m:])
# #         Mod_v = Signal(intbv(0)[m:])
        Mul = Signal(intbv(0, min=-2 ** 17, max=2 ** 17))
        Mul_v = Signal(intbv(0, min=-2 ** 17, max=2 ** 17))
# #         Pow = Signal(intbv(0)[64:])
# #         Pow_v = Signal(intbv(0)[64:])
        RightShift = Signal(intbv(0, min=-M, max=M))
        RightShift_v = Signal(intbv(0, min=-M, max=M))
        Sub = Signal(intbv(min=-M, max=M))
        Sub_v = Signal(intbv(min=-M, max=M))
        Sum, Sum1, Sum2, Sum3 = [Signal(intbv(min=-M, max=M)) for __ in range(4)]
        Sum_v, Sum1_v, Sum2_v, Sum3_v = [Signal(intbv(min=-M, max=M)) for __ in range(4)]
        EQ, NE, LT, GT, LE, GE = [Signal(bool()) for __ in range(6)]
        EQ_v, NE_v, LT_v, GT_v, LE_v, GE_v = [Signal(bool()) for __ in range(6)]
        And, Or = [Signal(bool()) for __ in range(2)]
        And_v, Or_v, = [Signal(bool()) for __ in range(2)]

        binops = binaryOps(
# #                            Bitand,
# #                            Bitor,
# #                            Bitxor,
# #                            FloorDiv,
                           LeftShift,
# #                            Mod,
                           Mul,
# #                            Pow,
                           RightShift,
                           Sub,
                           Sum, Sum1, Sum2, Sum3,
                           EQ,
                           NE,
                           LT,
                           GT,
                           LE,
                           GE,
                           And,
                           Or,
                           left, right, bit).convert(hdl='Verilog')
        binops_v = binaryOps_v(binaryOps.__name__,
# #                                Bitand_v,
# #                                Bitor_v,
# #                                Bitxor_v,
# #                                FloorDiv_v,
                               LeftShift_v,
# #                                Mod_v,
                               Mul_v,
# #                                Pow_v,
                               RightShift_v,
                               Sub_v,
                               Sum_v, Sum1_v, Sum2_v, Sum3_v,
                               EQ_v,
                               NE_v,
                               LT_v,
                               GT_v,
                               LE_v,
                               GE_v,
                               And_v,
                               Or_v,
                               left, right, bit)

        def stimulus():
            for dummy in range(100):
                # bit.next = False
                left.next = randrange(Ll, Ml)
                right.next = randrange(Lr, Mr)
                yield delay(10)
            for j, k in ((Ll, Lr), (Ml - 1, Mr - 1), (Ll, Mr - 1), (Ml - 1, Lr)):
                left.next = j
                right.next = k
                yield delay(10)

        def check():
            while 1:
                yield left, right
                bit.next = not bit
                yield delay(1)

                # print "%s %s %s %s" % (left, right, Mul, Mul_v)
                # print "%s %s %s %s" % (left, right, bin(Mul), bin(Mul_v))
                # print "%s %s %s %s" % (left, right, Sum, Sum_v)
                # print "%s %s %s %s" % (left, right, bin(Sum), bin(Sum_v))
# #                 print left
# #                 print right
# #                 print bin(left)
# #                 print bin(right)
# #                 print bin(Bitand)
# #                 print bin(Bitand_v)
# #                 print Bitand
# #                 print Bitand_v
# #                self.assertEqual(Bitand, Bitand_v)
                # w = len(Bitand)
                # self.assertEqual(bin(Bitand, w), bin(Bitand_v,w ))
# #                 self.assertEqual(Bitor, Bitor_v)
# #                 self.assertEqual(Bitxor, Bitxor_v)
# #                 self.assertEqual(FloorDiv, FloorDiv_v)
                self.assertEqual(LeftShift, LeftShift_v)
# #                 self.assertEqual(Mod, Mod_v)
                self.assertEqual(Mul, Mul_v)
                # self.assertEqual(Pow, Pow_v)
                self.assertEqual(RightShift, RightShift_v)
                self.assertEqual(Sub, Sub_v)
                self.assertEqual(Sum, Sum_v)
                self.assertEqual(Sum1, Sum1_v)
                self.assertEqual(Sum2, Sum2_v)
                self.assertEqual(Sum3, Sum3_v)
                self.assertEqual(EQ, EQ_v)
                self.assertEqual(NE, NE_v)
                self.assertEqual(LT, LT_v)
                self.assertEqual(GT, GT_v)
                self.assertEqual(LE, LE_v)
                self.assertEqual(GE, GE_v)
                self.assertEqual(And, And_v)
                self.assertEqual(Or, Or_v)

        return binops, binops_v, stimulus(), check()

    def testBinaryOps(self):
        for Ll, Ml, Lr, Mr in (
                                (-254, 236, 0, 4),
                                (-128, 128, -128, 128),
                                (-53, 25, -23, 123),
                                (-23, 145, -66, 12),
                                (23, 34, -34, -16),
                                (-54, -20, 45, 73),
                                (-25, -12, -123, -66),
                              ):
            sim = self.binaryBench(Ll, Ml, Lr, Mr)
            Simulation(sim).run()


@block
def unaryOps(
             Not,
             Invert,
             UnaryAdd,
             UnarySub,
             arg):

    @instance
    def comb():
        while 1:
            yield arg
            Not.next = not arg
            Invert.next = ~arg
            UnaryAdd.next = +arg
            UnarySub.next = - -arg

    return comb


def unaryOps_v(name,
               Not,
               Invert,
               UnaryAdd,
               UnarySub,
               arg):
    return setupCosimulation(**locals())


class TestUnaryOps(TestCase):

    def unaryBench(self, m):

        M = 2 ** m

        arg = Signal(intbv(0, min=-M, max=+M))
        Not = Signal(bool(0))
        Not_v = Signal(bool(0))
        Invert = Signal(intbv(0, min=-M, max=+M))
        Invert_v = Signal(intbv(0, min=-M, max=+M))
        UnaryAdd = Signal(intbv(0, min=-M, max=+M))
        UnaryAdd_v = Signal(intbv(0, min=-M, max=+M))
        UnarySub = Signal(intbv(0, min=-M, max=+M))
        UnarySub_v = Signal(intbv(0, min=-M, max=+M))

        unaryops = unaryOps(
                             Not,
                             Invert,
                             UnaryAdd,
                             UnarySub,
                             arg).convert(hdl='Verilog')
        unaryops_v = unaryOps_v(unaryOps.__name__,
                                Not_v,
                                Invert_v,
                                UnaryAdd_v,
                                UnarySub_v,
                                arg)

        def stimulus():
            for i in range(-M, M):
                arg.next = intbv(i)
                yield delay(10)
            for i in range(100):
                arg.next = randrange(-M, M)
                yield delay(10)
            raise StopSimulation

        def check():
            while 1:
                yield arg
                yield delay(1)
                self.assertEqual(Not, Not_v)
                self.assertEqual(Invert, Invert_v)
                self.assertEqual(UnaryAdd, UnaryAdd_v)
                self.assertEqual(UnarySub, UnarySub_v)

        return unaryops, unaryops_v, stimulus(), check()

    def testUnaryOps(self):
        for m in (4, 7):
            sim = self.unaryBench(m)
            Simulation(sim).run()


@block
def augmOps(
# #               Bitand,
# #               Bitor,
# #               Bitxor,
# #               FloorDiv,
              LeftShift,
# #               Mod,
              Mul,
              RightShift,
              Sub,
              Sum,
              left, right):

    @instance
    def comb():
        var = intbv(0, min=-2 ** 17, max=+2 ** 17)
        var2 = intbv(0, min=-2 ** 64, max=+2 ** 64)
        while 1:
            yield left, right
    # #         var[:] = left
    # #         var &= right
    # #         Bitand.next = var
    # #         var[:] = left
    # #         var |= right
    # #         Bitor.next = var
    # #         var[:] = left
    # #         var ^= left
    # #         Bitxor.next = var
    # #         if right != 0:
    # #             var[:] = left
    # #             var //= right
    # #             FloorDiv.next = var
            if left < 256 and right < 40 and right >= 0:
                var2[:] = left
                var2 <<= right
                LeftShift.next = var2
    # #         if right != 0:
    # #             var[:] = left
    # #             var %= right
    # #             Mod.next = var
            var[:] = left
            var *= right
            Mul.next = var

            var[:] = left
            if right >= 0:
                var >>= right
                RightShift.next = var

            var[:] = left
            var -= right
            Sub.next = var
            var[:] = left
            var += right
            Sum.next = var

    return comb


@block
def augmOps_v(name,
# #                 Bitand,
# #                 Bitor,
# #                 Bitxor,
# #                 FloorDiv,
                LeftShift,
# #                 Mod,
                Mul,
                RightShift,
                Sub,
                Sum,
                left, right):
    return setupCosimulation(**locals())


class TestAugmOps(TestCase):

    def augmBench(self, Ll, Ml, Lr, Mr):

        left = Signal(intbv(Ll, min=Ll, max=Ml))
        right = Signal(intbv(Lr, min=Lr, max=Mr))
        M = 2 ** 17

# #         Bitand = Signal(intbv(0)[max(m, n):])
# #         Bitand_v = Signal(intbv(0)[max(m, n):])
# #         Bitor = Signal(intbv(0)[max(m, n):])
# #         Bitor_v = Signal(intbv(0)[max(m, n):])
# #         Bitxor = Signal(intbv(0)[max(m, n):])
# #         Bitxor_v = Signal(intbv(0)[max(m, n):])

# #         FloorDiv = Signal(intbv(0)[m:])
# #         FloorDiv_v = Signal(intbv(0)[m:])
        LeftShift = Signal(intbv(0, min=-2 ** 64, max=2 ** 64))
        LeftShift_v = Signal(intbv(0, min=-2 ** 64, max=2 ** 64))
# #         Mod = Signal(intbv(0)[m:])
# #         Mod_v = Signal(intbv(0)[m:])

        Mul = Signal(intbv(0, min=-M, max=+M))
        Mul_v = Signal(intbv(0, min=-M, max=+M))

        RightShift = Signal(intbv(0, min=-M, max=+M))
        RightShift_v = Signal(intbv(0, min=-M, max=+M))

        Sub = Signal(intbv(0, min=-M, max=+M))
        Sub_v = Signal(intbv(0, min=-M, max=+M))
        Sum = Signal(intbv(0, min=-M, max=+M))
        Sum_v = Signal(intbv(0, min=-M, max=+M))

        augmops = augmOps(
# #                            Bitand,
# #                            Bitor,
# #                            Bitxor,
# #                            FloorDiv,
                           LeftShift,
# #                            Mod,
                           Mul,
                           RightShift,
                           Sub,
                           Sum,
                           left,
                           right).convert(hdl='Verilog')
        augmops_v = augmOps_v(augmOps.__name__,
# #                                Bitand_v,
# #                                Bitor_v,
# #                                Bitxor_v,
# #                                FloorDiv_v,
                               LeftShift_v,
# #                                Mod_v,
                               Mul_v,
                               RightShift_v,
                               Sub_v,
                               Sum_v,
                               left, right)

        def stimulus():
            for dummy in range(100):
                left.next = randrange(Ll, Ml)
                right.next = randrange(Lr, Mr)
                yield delay(10)
            for j, k in ((Ll, Lr), (Ml - 1, Mr - 1), (Ll, Mr - 1), (Ml - 1, Lr)):
                left.next = j
                right.next = k
                yield delay(10)

        def check():
            while 1:
                yield left, right
                yield delay(1)
                # print "%s %s %s %s" % (left, right, Or, Or_v)
# #                 self.assertEqual(Bitand, Bitand_v)
# #                 self.assertEqual(Bitor, Bitor_v)
# #                 self.assertEqual(Bitxor, Bitxor_v)
# #                 self.assertEqual(FloorDiv, FloorDiv_v)
# #                 self.assertEqual(LeftShift, LeftShift_v)
# #                 self.assertEqual(Mod, Mod_v)
                self.assertEqual(Mul, Mul_v)
                self.assertEqual(RightShift, RightShift_v)
                self.assertEqual(Sub, Sub_v)
                self.assertEqual(Sum, Sum_v)

        return augmops, augmops_v, stimulus(), check()

    def testAugmOps(self):
        for Ll, Ml, Lr, Mr in (
                                (-254, 236, 0, 4),
                                (-128, 128, -128, 128),
                                (-53, 25, -23, 123),
                                (-23, 145, -66, 12),
                                (23, 34, -34, -16),
                                (-54, -20, 45, 73),
                                (-25, -12, -123, -66),
                              ):
            sim = self.augmBench(Ll, Ml, Lr, Mr)
            Simulation(sim).run()


@block
def expressions(a, b, clk):

    c = Signal(intbv(0, min=0, max=47))

    @instance
    def comb():

        d = intbv(0, min=-23, max=43)
        d[:] = -17

        c.next = 5
        yield clk.posedge
        a.next = c + 1
        b.next = c + 1
        yield clk.posedge
        a.next = c + -10
        b.next = c + -1
        yield clk.posedge
        a.next = c < -10
        b.next = c < -1
        yield clk.posedge
        a.next = d + c
        b.next = d >= c
        yield clk.posedge
        # a.next = d & c
        # b.next = c + (d & c)
        yield clk.posedge
        a.next = d + -c
        b.next = c + (-d)
        yield clk.posedge
        a.next = -d
        yield clk.posedge
        a.next = -c
        yield clk.posedge
        c.next = 46
        yield clk.posedge
        a.next = ~d + 1
        b.next = ~c + 1
        yield clk.posedge
        a.next = ~c + 1
        b.next = ~d + 1

        yield clk.posedge
        raise StopSimulation

    return comb


@block
def expressions_v(a, b, clk):
    return setupCosimulation(**locals())


class TestExpressions(TestCase):

    @block
    def bench(self):

        a, a_v = [Signal(intbv(0, min=-34, max=47)) for __ in range(2)]
        b, b_v = [Signal(intbv(0, min=0, max=47)) for __ in range(2)]
        clk = Signal(bool())

        expr = expressions(a, b, clk).convert(hdl='Verilog')
        expr_v = expressions(a_v, b_v, clk).convert(hdl='Verilog')

        @instance
        def check():
            while 1:
                yield clk.posedge
                yield delay(1)
                self.assertEqual(a, a_v)
                # print a
                # print a_v
                self.assertEqual(b, b_v)

        @always(delay(10))
        def clkgen():
            clk.next = not clk

        return expr, expr_v, check, clkgen

    def testExpressions(self):
        sim = self.bench()
        Simulation(sim).run()


if __name__ == '__main__':
    unittest.main()




import myhdl
from myhdl import *

ACTIVE_LOW, INACTIVE_HIGH = 0, 1


def dff(q, d, clk, reset):
    """ D flip-flop.
    
    q -- output
    d -- input 
    clock -- clock input
    reset -- asynchronous reset input
    """

    @always(clk.posedge, reset.negedge)
    def synch():
        if reset == ACTIVE_LOW:
            q.next = 0
        else:
            q.next = d

    return synch

